@@ -1,8 +1,5 @@
 #!/usr/bin/env bash
 
-# TODO: add sparse back in, once Numba works with the development version of
-# NumPy again: https://github.com/pydata/xarray/issues/4146
-
 conda uninstall -y --force \
     numpy \
     scipy \
@@ -10,12 +7,16 @@ conda uninstall -y --force \
     matplotlib \
     dask \
     distributed \
+    fsspec \
     zarr \
     cftime \
     rasterio \
+    packaging \
     pint \
-    bottleneck \
-    sparse
+    sparse \
+    h5netcdf \
+    xarray
+    # bottleneck \ # re-enable again, see https://github.com/pydata/bottleneck/pull/378
 # to limit the runtime of Upstream CI
 python -m pip install pytest-timeout
 python -m pip install \
@@ -40,6 +41,10 @@ python -m pip install \
     git+https://github.com/zarr-developers/zarr \
     git+https://github.com/Unidata/cftime \
     git+https://github.com/mapbox/rasterio \
+    git+https://github.com/pypa/packaging \
     git+https://github.com/hgrecco/pint \
-    git+https://github.com/pydata/bottleneck # \
-    # git+https://github.com/pydata/sparse
+    git+https://github.com/pydata/sparse \
+    git+https://github.com/intake/filesystem_spec \
+    git+https://github.com/SciTools/nc-time-axis \
+    git+https://github.com/h5netcdf/h5netcdf
+    # git+https://github.com/pydata/bottleneck \ # re-enable again, see https://github.com/pydata/bottleneck/pull/378
