@@ -7,7 +7,7 @@ import sys
 from datetime import datetime
 from typing import Dict, Iterator, Optional, Tuple
 
-import conda.api
+import conda.api  # type: ignore[import]
 import yaml
 from dateutil.relativedelta import relativedelta
 
@@ -20,24 +20,16 @@ IGNORE_DEPS = {
     "isort",
     "mypy",
     "pip",
+    "setuptools",
     "pytest",
     "pytest-cov",
     "pytest-env",
     "pytest-xdist",
 }
 
-POLICY_MONTHS = {"python": 24, "numpy": 18, "setuptools": 42}
+POLICY_MONTHS = {"python": 24, "numpy": 18}
 POLICY_MONTHS_DEFAULT = 12
-POLICY_OVERRIDE = {
-    # setuptools-scm doesn't work with setuptools < 36.7 (Nov 2017).
-    # The conda metadata is malformed for setuptools < 38.4 (Jan 2018)
-    # (it's missing a timestamp which prevents this tool from working).
-    # setuptools < 40.4 (Sep 2018) from conda-forge cannot be installed into a py37
-    # environment
-    # TODO remove this special case and the matching note in installing.rst
-    #      after March 2022.
-    "setuptools": (40, 4),
-}
+POLICY_OVERRIDE: Dict[str, Tuple[int, int]] = {}
 has_errors = False
 
 
@@ -76,9 +68,9 @@ def parse_requirements(fname) -> Iterator[Tuple[str, int, int, Optional[int]]]:
             raise ValueError("non-numerical version: " + row)
 
         if len(version_tup) == 2:
-            yield (pkg, *version_tup, None)  # type: ignore
+            yield (pkg, *version_tup, None)  # type: ignore[misc]
         elif len(version_tup) == 3:
-            yield (pkg, *version_tup)  # type: ignore
+            yield (pkg, *version_tup)  # type: ignore[misc]
         else:
             raise ValueError("expected major.minor or major.minor.patch: " + row)
 
