@@ -1,5 +1,3 @@
-import pkg_resources
-
 from . import testing, tutorial, ufuncs
 from .backends.api import (
     load_dataarray,
@@ -11,26 +9,41 @@ from .backends.api import (
 )
 from .backends.rasterio_ import open_rasterio
 from .backends.zarr import open_zarr
-from .coding.cftime_offsets import cftime_range
+from .coding.cftime_offsets import cftime_range, date_range, date_range_like
 from .coding.cftimeindex import CFTimeIndex
 from .coding.frequencies import infer_freq
 from .conventions import SerializationWarning, decode_cf
 from .core.alignment import align, broadcast
 from .core.combine import combine_by_coords, combine_nested
 from .core.common import ALL_DIMS, full_like, ones_like, zeros_like
-from .core.computation import apply_ufunc, corr, cov, dot, polyval, where
+from .core.computation import (
+    apply_ufunc,
+    corr,
+    cov,
+    cross,
+    dot,
+    polyval,
+    unify_chunks,
+    where,
+)
 from .core.concat import concat
 from .core.dataarray import DataArray
 from .core.dataset import Dataset
 from .core.extensions import register_dataarray_accessor, register_dataset_accessor
-from .core.merge import MergeError, merge
-from .core.options import set_options
+from .core.merge import Context, MergeError, merge
+from .core.options import get_options, set_options
 from .core.parallel import map_blocks
 from .core.variable import Coordinate, IndexVariable, Variable, as_variable
 from .util.print_versions import show_versions
 
 try:
-    __version__ = pkg_resources.get_distribution("xarray").version
+    from importlib.metadata import version as _version
+except ImportError:
+    # if the fallback library is missing, we are doomed.
+    from importlib_metadata import version as _version  # type: ignore[no-redef]
+
+try:
+    __version__ = _version("xarray")
 except Exception:
     # Local copy or not installed with setuptools.
     # Disable minimum version checks on downstream libraries.
@@ -52,11 +65,15 @@ __all__ = (
     "combine_by_coords",
     "combine_nested",
     "concat",
+    "date_range",
+    "date_range_like",
     "decode_cf",
     "dot",
     "cov",
     "corr",
+    "cross",
     "full_like",
+    "get_options",
     "infer_freq",
     "load_dataarray",
     "load_dataset",
@@ -74,10 +91,12 @@ __all__ = (
     "save_mfdataset",
     "set_options",
     "show_versions",
+    "unify_chunks",
     "where",
     "zeros_like",
     # Classes
     "CFTimeIndex",
+    "Context",
     "Coordinate",
     "DataArray",
     "Dataset",
