@@ -2,7 +2,7 @@ import contextlib
 import io
 import threading
 import warnings
-from typing import Any, Dict, cast
+from typing import Any, Dict
 
 from ..core import utils
 from ..core.options import OPTIONS
@@ -11,7 +11,7 @@ from .lru_cache import LRUCache
 
 # Global cache for storing open files.
 FILE_CACHE: LRUCache[str, io.IOBase] = LRUCache(
-    maxsize=cast(int, OPTIONS["file_cache_maxsize"]), on_evict=lambda k, v: v.close()
+    maxsize=OPTIONS["file_cache_maxsize"], on_evict=lambda k, v: v.close()
 )
 assert FILE_CACHE.maxsize, "file cache must be at least size one"
 
@@ -175,7 +175,8 @@ class CachingFileManager(FileManager):
 
         Returns
         -------
-        An open file object, as returned by ``opener(*args, **kwargs)``.
+        file-like
+            An open file object, as returned by ``opener(*args, **kwargs)``.
         """
         file, _ = self._acquire_with_cache_info(needs_lock)
         return file
@@ -313,8 +314,7 @@ class _HashedSequence(list):
 
 
 class DummyFileManager(FileManager):
-    """FileManager that simply wraps an open file in the FileManager interface.
-    """
+    """FileManager that simply wraps an open file in the FileManager interface."""
 
     def __init__(self, value):
         self._value = value
