@@ -1,14 +1,25 @@
 import functools
-from distutils.version import LooseVersion
+import io
+import os
 
 import numpy as np
+from packaging.version import Version
 
-from .. import conventions
 from ..core import indexing
-from ..core.dataset import Dataset
-from ..core.utils import FrozenDict, close_on_error, is_remote_uri
+from ..core.utils import (
+    FrozenDict,
+    is_remote_uri,
+    read_magic_number_from_file,
+    try_read_magic_number_from_file_or_path,
+)
 from ..core.variable import Variable
-from .common import WritableCFDataStore, find_root_and_group
+from .common import (
+    BACKEND_ENTRYPOINTS,
+    BackendEntrypoint,
+    WritableCFDataStore,
+    _normalize_path,
+    find_root_and_group,
+)
 from .file_manager import CachingFileManager, DummyFileManager
 from .locks import HDF5_LOCK, combine_locks, ensure_lock, get_write_lock
 from .netCDF4_ import (
@@ -18,13 +29,23 @@ from .netCDF4_ import (
     _get_datatype,
     _nc4_require_group,
 )
+from .store import StoreBackendEntrypoint
+
+try:
+    import h5netcdf
+
+    has_h5netcdf = True
+except ImportError:
+    # Except a base ImportError (not ModuleNotFoundError) to catch usecases
+    # where errors have mismatched versions of c-dependencies. This can happen
+    # when developers are making changes them.
+    has_h5netcdf = False
 
 
 class H5NetCDFArrayWrapper(BaseNetCDF4Array):
     def get_array(self, needs_lock=True):
         ds = self.datastore._acquire(needs_lock)
-        variable = ds.variables[self.variable_name]
-        return variable
+        return ds.variables[self.variable_name]
 
     def __getitem__(self, key):
         return indexing.explicit_indexing_adapter(
@@ -60,7 +81,11 @@ def _read_attributes(h5netcdf_var):
 
 
 _extract_h5nc_encoding = functools.partial(
-    _extract_nc4_variable_encoding, lsd_okay=False, h5py_okay=True, backend="h5netcdf"
+    _extract_nc4_variable_encoding,
+    lsd_okay=False,
+    h5py_okay=True,
+    backend="h5netcdf",
+    unlimited_dims=None,
 )
 
 
@@ -84,13 +109,11 @@ class H5NetCDFStore(WritableCFDataStore):
 
     def __init__(self, manager, group=None, mode=None, lock=HDF5_LOCK, autoclose=False):
 
-        import h5netcdf
-
         if isinstance(manager, (h5netcdf.File, h5netcdf.Group)):
             if group is None:
                 root, group = find_root_and_group(manager)
             else:
-                if not type(manager) is h5netcdf.File:
+                if type(manager) is not h5netcdf.File:
                     raise ValueError(
                         "must supply a h5netcdf.File if the group "
                         "argument is provided"
@@ -120,40 +143,31 @@ class H5NetCDFStore(WritableCFDataStore):
         autoclose=False,
         invalid_netcdf=None,
         phony_dims=None,
+        decode_vlen_strings=True,
     ):
-        import h5netcdf
 
         if isinstance(filename, bytes):
             raise ValueError(
                 "can't open netCDF4/HDF5 as bytes "
                 "try passing a path or file-like object"
             )
-        elif hasattr(filename, "tell"):
-            if filename.tell() != 0:
+        elif isinstance(filename, io.IOBase):
+            magic_number = read_magic_number_from_file(filename)
+            if not magic_number.startswith(b"\211HDF\r\n\032\n"):
                 raise ValueError(
-                    "file-like object read/write pointer not at zero "
-                    "please close and reopen, or use a context manager"
+                    f"{magic_number} is not the signature of a valid netCDF4 file"
                 )
-            else:
-                magic_number = filename.read(8)
-                filename.seek(0)
-                if not magic_number.startswith(b"\211HDF\r\n\032\n"):
-                    raise ValueError(
-                        f"{magic_number} is not the signature of a valid netCDF file"
-                    )
 
         if format not in [None, "NETCDF4"]:
             raise ValueError("invalid format for h5netcdf backend")
 
         kwargs = {"invalid_netcdf": invalid_netcdf}
         if phony_dims is not None:
-            if LooseVersion(h5netcdf.__version__) >= LooseVersion("0.8.0"):
-                kwargs["phony_dims"] = phony_dims
-            else:
-                raise ValueError(
-                    "h5netcdf backend keyword argument 'phony_dims' needs "
-                    "h5netcdf >= 0.8.0."
-                )
+            kwargs["phony_dims"] = phony_dims
+        if Version(h5netcdf.__version__) >= Version("0.10.0") and Version(
+            h5netcdf.core.h5py.__version__
+        ) >= Version("3.0.0"):
+            kwargs["decode_vlen_strings"] = decode_vlen_strings
 
         if lock is None:
             if mode == "r":
@@ -179,7 +193,7 @@ class H5NetCDFStore(WritableCFDataStore):
         import h5py
 
         dimensions = var.dimensions
-        data = indexing.LazilyOuterIndexedArray(H5NetCDFArrayWrapper(name, self))
+        data = indexing.LazilyIndexedArray(H5NetCDFArrayWrapper(name, self))
         attrs = _read_attributes(var)
 
         # netCDF4 specific encoding
@@ -221,14 +235,24 @@ class H5NetCDFStore(WritableCFDataStore):
         return FrozenDict(_read_attributes(self.ds))
 
     def get_dimensions(self):
-        return self.ds.dimensions
+        if Version(h5netcdf.__version__) >= Version("0.14.0.dev0"):
+            return FrozenDict((k, len(v)) for k, v in self.ds.dimensions.items())
+        else:
+            return self.ds.dimensions
 
     def get_encoding(self):
-        encoding = {}
-        encoding["unlimited_dims"] = {
-            k for k, v in self.ds.dimensions.items() if v is None
-        }
-        return encoding
+        if Version(h5netcdf.__version__) >= Version("0.14.0.dev0"):
+            return {
+                "unlimited_dims": {
+                    k for k, v in self.ds.dimensions.items() if v.isunlimited()
+                }
+            }
+        else:
+            return {
+                "unlimited_dims": {
+                    k for k, v in self.ds.dimensions.items() if v is None
+                }
+            }
 
     def set_dimension(self, name, length, is_unlimited=False):
         if is_unlimited:
@@ -257,9 +281,9 @@ class H5NetCDFStore(WritableCFDataStore):
                 "h5netcdf does not yet support setting a fill value for "
                 "variable-length strings "
                 "(https://github.com/shoyer/h5netcdf/issues/37). "
-                "Either remove '_FillValue' from encoding on variable %r "
+                f"Either remove '_FillValue' from encoding on variable {name!r} "
                 "or set {'dtype': 'S1'} in encoding to use the fixed width "
-                "NC_CHAR type." % name
+                "NC_CHAR type."
             )
 
         if dtype is str:
@@ -325,40 +349,55 @@ class H5NetCDFStore(WritableCFDataStore):
         self._manager.close(**kwargs)
 
 
-def open_backend_dataset_h5necdf(
-    filename_or_obj,
-    *,
-    mask_and_scale=True,
-    decode_times=None,
-    concat_characters=None,
-    decode_coords=None,
-    drop_variables=None,
-    use_cftime=None,
-    decode_timedelta=None,
-    format=None,
-    group=None,
-    lock=None,
-    invalid_netcdf=None,
-    phony_dims=None,
-):
+class H5netcdfBackendEntrypoint(BackendEntrypoint):
+    available = has_h5netcdf
 
-    store = H5NetCDFStore.open(
+    def guess_can_open(self, filename_or_obj):
+        magic_number = try_read_magic_number_from_file_or_path(filename_or_obj)
+        if magic_number is not None:
+            return magic_number.startswith(b"\211HDF\r\n\032\n")
+
+        try:
+            _, ext = os.path.splitext(filename_or_obj)
+        except TypeError:
+            return False
+
+        return ext in {".nc", ".nc4", ".cdf"}
+
+    def open_dataset(
+        self,
         filename_or_obj,
-        format=format,
-        group=group,
-        lock=lock,
-        invalid_netcdf=invalid_netcdf,
-        phony_dims=phony_dims,
-    )
+        *,
+        mask_and_scale=True,
+        decode_times=True,
+        concat_characters=True,
+        decode_coords=True,
+        drop_variables=None,
+        use_cftime=None,
+        decode_timedelta=None,
+        format=None,
+        group=None,
+        lock=None,
+        invalid_netcdf=None,
+        phony_dims=None,
+        decode_vlen_strings=True,
+    ):
 
-    with close_on_error(store):
-        vars, attrs = store.load()
-        file_obj = store
-        encoding = store.get_encoding()
+        filename_or_obj = _normalize_path(filename_or_obj)
+        store = H5NetCDFStore.open(
+            filename_or_obj,
+            format=format,
+            group=group,
+            lock=lock,
+            invalid_netcdf=invalid_netcdf,
+            phony_dims=phony_dims,
+            decode_vlen_strings=decode_vlen_strings,
+        )
 
-        vars, attrs, coord_names = conventions.decode_cf_variables(
-            vars,
-            attrs,
+        store_entrypoint = StoreBackendEntrypoint()
+
+        ds = store_entrypoint.open_dataset(
+            store,
             mask_and_scale=mask_and_scale,
             decode_times=decode_times,
             concat_characters=concat_characters,
@@ -367,10 +406,7 @@ def open_backend_dataset_h5necdf(
             use_cftime=use_cftime,
             decode_timedelta=decode_timedelta,
         )
+        return ds
 
-        ds = Dataset(vars, attrs=attrs)
-        ds = ds.set_coords(coord_names.intersection(vars))
-        ds._file_obj = file_obj
-        ds.encoding = encoding
 
-    return ds
+BACKEND_ENTRYPOINTS["h5netcdf"] = H5netcdfBackendEntrypoint
