@@ -52,9 +52,9 @@ class RasterioArrayWrapper(BackendArray):
     def _get_indexer(self, key):
         """Get indexer for rasterio array.
 
-        Parameter
-        ---------
-        key: tuple of int
+        Parameters
+        ----------
+        key : tuple of int
 
         Returns
         -------
@@ -63,7 +63,7 @@ class RasterioArrayWrapper(BackendArray):
         squeeze_axis: axes to be squeezed
         np_ind: indexer for loaded numpy array
 
-        See also
+        See Also
         --------
         indexing.decompose_indexer
         """
@@ -162,8 +162,21 @@ def _parse_envi(meta):
     return parsed_meta
 
 
-def open_rasterio(filename, parse_coordinates=None, chunks=None, cache=None, lock=None):
-    """Open a file with rasterio (experimental).
+def open_rasterio(
+    filename,
+    parse_coordinates=None,
+    chunks=None,
+    cache=None,
+    lock=None,
+    **kwargs,
+):
+    """Open a file with rasterio.
+
+    .. deprecated:: 0.20.0
+
+        Deprecated in favor of rioxarray.
+        For information about transitioning, see:
+        https://corteva.github.io/rioxarray/stable/getting_started/getting_started.html
 
     This should work with any file that rasterio can open (most often:
     geoTIFF). The x and y coordinates are generated automatically from the
@@ -174,12 +187,46 @@ def open_rasterio(filename, parse_coordinates=None, chunks=None, cache=None, loc
 
     You can generate 2D coordinates from the file's attributes with::
 
-        from affine import Affine
-        da = xr.open_rasterio('path_to_file.tif')
-        transform = Affine.from_gdal(*da.attrs['transform'])
-        nx, ny = da.sizes['x'], da.sizes['y']
-        x, y = np.meshgrid(np.arange(nx)+0.5, np.arange(ny)+0.5) * transform
-
+        >>> from affine import Affine
+        >>> da = xr.open_rasterio(
+        ...     "https://github.com/mapbox/rasterio/raw/1.2.1/tests/data/RGB.byte.tif"
+        ... )
+        >>> da
+        <xarray.DataArray (band: 3, y: 718, x: 791)>
+        [1703814 values with dtype=uint8]
+        Coordinates:
+          * band     (band) int64 1 2 3
+          * y        (y) float64 2.827e+06 2.826e+06 2.826e+06 ... 2.612e+06 2.612e+06
+          * x        (x) float64 1.021e+05 1.024e+05 1.027e+05 ... 3.389e+05 3.392e+05
+        Attributes:
+            transform:      (300.0379266750948, 0.0, 101985.0, 0.0, -300.041782729805...
+            crs:            +init=epsg:32618
+            res:            (300.0379266750948, 300.041782729805)
+            is_tiled:       0
+            nodatavals:     (0.0, 0.0, 0.0)
+            scales:         (1.0, 1.0, 1.0)
+            offsets:        (0.0, 0.0, 0.0)
+            AREA_OR_POINT:  Area
+        >>> transform = Affine(*da.attrs["transform"])
+        >>> transform
+        Affine(300.0379266750948, 0.0, 101985.0,
+               0.0, -300.041782729805, 2826915.0)
+        >>> nx, ny = da.sizes["x"], da.sizes["y"]
+        >>> x, y = transform * np.meshgrid(np.arange(nx) + 0.5, np.arange(ny) + 0.5)
+        >>> x
+        array([[102135.01896334, 102435.05689001, 102735.09481669, ...,
+                338564.90518331, 338864.94310999, 339164.98103666],
+               [102135.01896334, 102435.05689001, 102735.09481669, ...,
+                338564.90518331, 338864.94310999, 339164.98103666],
+               [102135.01896334, 102435.05689001, 102735.09481669, ...,
+                338564.90518331, 338864.94310999, 339164.98103666],
+               ...,
+               [102135.01896334, 102435.05689001, 102735.09481669, ...,
+                338564.90518331, 338864.94310999, 339164.98103666],
+               [102135.01896334, 102435.05689001, 102735.09481669, ...,
+                338564.90518331, 338864.94310999, 339164.98103666],
+               [102135.01896334, 102435.05689001, 102735.09481669, ...,
+                338564.90518331, 338864.94310999, 339164.98103666]])
 
     Parameters
     ----------
@@ -211,6 +258,13 @@ def open_rasterio(filename, parse_coordinates=None, chunks=None, cache=None, loc
     data : DataArray
         The newly created DataArray.
     """
+    warnings.warn(
+        "open_rasterio is Deprecated in favor of rioxarray. "
+        "For information about transitioning, see: "
+        "https://corteva.github.io/rioxarray/stable/getting_started/getting_started.html",
+        DeprecationWarning,
+        stacklevel=2,
+    )
     import rasterio
     from rasterio.vrt import WarpedVRT
 
@@ -238,7 +292,13 @@ def open_rasterio(filename, parse_coordinates=None, chunks=None, cache=None, loc
     if lock is None:
         lock = RASTERIO_LOCK
 
-    manager = CachingFileManager(rasterio.open, filename, lock=lock, mode="r")
+    manager = CachingFileManager(
+        rasterio.open,
+        filename,
+        lock=lock,
+        mode="r",
+        kwargs=kwargs,
+    )
     riods = manager.acquire()
     if vrt_params is not None:
         riods = WarpedVRT(riods, **vrt_params)
@@ -336,9 +396,7 @@ def open_rasterio(filename, parse_coordinates=None, chunks=None, cache=None, loc
             else:
                 attrs[k] = v
 
-    data = indexing.LazilyOuterIndexedArray(
-        RasterioArrayWrapper(manager, lock, vrt_params)
-    )
+    data = indexing.LazilyIndexedArray(RasterioArrayWrapper(manager, lock, vrt_params))
 
     # this lets you write arrays loaded with rasterio
     data = indexing.CopyOnWriteArray(data)
@@ -357,10 +415,10 @@ def open_rasterio(filename, parse_coordinates=None, chunks=None, cache=None, loc
             # the filename is probably an s3 bucket rather than a regular file
             mtime = None
         token = tokenize(filename, mtime, chunks)
-        name_prefix = "open_rasterio-%s" % token
+        name_prefix = f"open_rasterio-{token}"
         result = result.chunk(chunks, name_prefix=name_prefix, token=token)
 
     # Make the file closeable
-    result._file_obj = manager
+    result.set_close(manager.close)
 
     return result
