@@ -1,15 +1,6 @@
-from typing import (
-    TYPE_CHECKING,
-    Dict,
-    Hashable,
-    Iterable,
-    List,
-    Optional,
-    Set,
-    Tuple,
-    Union,
-    overload,
-)
+from __future__ import annotations
+
+from typing import TYPE_CHECKING, Hashable, Iterable, Literal, overload
 
 import pandas as pd
 
@@ -24,34 +15,39 @@ if TYPE_CHECKING:
     from .dataarray import DataArray
     from .dataset import Dataset
 
+compat_options = Literal[
+    "identical", "equals", "broadcast_equals", "no_conflicts", "override"
+]
+concat_options = Literal["all", "minimal", "different"]
+
 
 @overload
 def concat(
-    objs: Iterable["Dataset"],
-    dim: Union[str, "DataArray", pd.Index],
-    data_vars: Union[str, List[str]] = "all",
-    coords: Union[str, List[str]] = "different",
-    compat: str = "equals",
-    positions: Optional[Iterable[int]] = None,
+    objs: Iterable[Dataset],
+    dim: Hashable | DataArray | pd.Index,
+    data_vars: concat_options | list[Hashable] = "all",
+    coords: concat_options | list[Hashable] = "different",
+    compat: compat_options = "equals",
+    positions: Iterable[int] | None = None,
     fill_value: object = dtypes.NA,
     join: str = "outer",
     combine_attrs: str = "override",
-) -> "Dataset":
+) -> Dataset:
     ...
 
 
 @overload
 def concat(
-    objs: Iterable["DataArray"],
-    dim: Union[str, "DataArray", pd.Index],
-    data_vars: Union[str, List[str]] = "all",
-    coords: Union[str, List[str]] = "different",
-    compat: str = "equals",
-    positions: Optional[Iterable[int]] = None,
+    objs: Iterable[DataArray],
+    dim: Hashable | DataArray | pd.Index,
+    data_vars: concat_options | list[Hashable] = "all",
+    coords: concat_options | list[Hashable] = "different",
+    compat: compat_options = "equals",
+    positions: Iterable[int] | None = None,
     fill_value: object = dtypes.NA,
     join: str = "outer",
     combine_attrs: str = "override",
-) -> "DataArray":
+) -> DataArray:
     ...
 
 
@@ -74,14 +70,14 @@ def concat(
         xarray objects to concatenate together. Each object is expected to
         consist of variables and coordinates with matching shapes except for
         along the concatenated dimension.
-    dim : str or DataArray or pandas.Index
+    dim : Hashable or DataArray or pandas.Index
         Name of the dimension to concatenate along. This can either be a new
         dimension name, in which case it is added along axis=0, or an existing
         dimension name, in which case the location of the dimension is
         unchanged. If dimension is provided as a DataArray or Index, its name
         is used as the dimension to concatenate along and the values are added
         as a coordinate.
-    data_vars : {"minimal", "different", "all"} or list of str, optional
+    data_vars : {"minimal", "different", "all"} or list of Hashable, optional
         These data variables will be concatenated together:
           * "minimal": Only data variables in which the dimension already
             appears are included.
@@ -91,11 +87,11 @@ def concat(
             load the data payload of data variables into memory if they are not
             already loaded.
           * "all": All data variables will be concatenated.
-          * list of str: The listed data variables will be concatenated, in
+          * list of dims: The listed data variables will be concatenated, in
             addition to the "minimal" data variables.
 
         If objects are DataArrays, data_vars must be "all".
-    coords : {"minimal", "different", "all"} or list of str, optional
+    coords : {"minimal", "different", "all"} or list of Hashable, optional
         These coordinate variables will be concatenated together:
           * "minimal": Only coordinates in which the dimension already appears
             are included.
@@ -106,7 +102,7 @@ def concat(
             loaded.
           * "all": All coordinate variables will be concatenated, except
             those corresponding to other dimensions.
-          * list of str: The listed coordinate variables will be concatenated,
+          * list of Hashable: The listed coordinate variables will be concatenated,
             in addition to the "minimal" coordinates.
     compat : {"identical", "equals", "broadcast_equals", "no_conflicts", "override"}, optional
         String indicating how to compare non-concatenated variables of the same name for
@@ -142,17 +138,23 @@ def concat(
         - "override": if indexes are of same size, rewrite indexes to be
           those of the first object with that dimension. Indexes for the same
           dimension must have the same size in all objects.
-    combine_attrs : {"drop", "identical", "no_conflicts", "override"}, \
-                    default: "override"
-        String indicating how to combine attrs of the objects being merged:
+    combine_attrs : {"drop", "identical", "no_conflicts", "drop_conflicts", \
+                     "override"} or callable, default: "override"
+        A callable or a string indicating how to combine attrs of the objects being
+        merged:
 
         - "drop": empty attrs on returned Dataset.
         - "identical": all attrs must be the same on every object.
         - "no_conflicts": attrs from all objects are combined, any that have
           the same name must also have the same value.
+        - "drop_conflicts": attrs from all objects are combined, any that have
+          the same name but different values are dropped.
         - "override": skip comparing and copy attrs from the first dataset to
           the result.
 
+        If a callable, it must expect a sequence of ``attrs`` dicts and a context object
+        as its only parameters.
+
     Returns
     -------
     concatenated : type of objs
@@ -160,7 +162,53 @@ def concat(
     See also
     --------
     merge
-    auto_combine
+
+    Examples
+    --------
+    >>> da = xr.DataArray(
+    ...     np.arange(6).reshape(2, 3), [("x", ["a", "b"]), ("y", [10, 20, 30])]
+    ... )
+    >>> da
+    <xarray.DataArray (x: 2, y: 3)>
+    array([[0, 1, 2],
+           [3, 4, 5]])
+    Coordinates:
+      * x        (x) <U1 'a' 'b'
+      * y        (y) int64 10 20 30
+
+    >>> xr.concat([da.isel(y=slice(0, 1)), da.isel(y=slice(1, None))], dim="y")
+    <xarray.DataArray (x: 2, y: 3)>
+    array([[0, 1, 2],
+           [3, 4, 5]])
+    Coordinates:
+      * x        (x) <U1 'a' 'b'
+      * y        (y) int64 10 20 30
+
+    >>> xr.concat([da.isel(x=0), da.isel(x=1)], "x")
+    <xarray.DataArray (x: 2, y: 3)>
+    array([[0, 1, 2],
+           [3, 4, 5]])
+    Coordinates:
+      * x        (x) <U1 'a' 'b'
+      * y        (y) int64 10 20 30
+
+    >>> xr.concat([da.isel(x=0), da.isel(x=1)], "new_dim")
+    <xarray.DataArray (new_dim: 2, y: 3)>
+    array([[0, 1, 2],
+           [3, 4, 5]])
+    Coordinates:
+        x        (new_dim) <U1 'a' 'b'
+      * y        (y) int64 10 20 30
+    Dimensions without coordinates: new_dim
+
+    >>> xr.concat([da.isel(x=0), da.isel(x=1)], pd.Index([-90, -100], name="new_dim"))
+    <xarray.DataArray (new_dim: 2, y: 3)>
+    array([[0, 1, 2],
+           [3, 4, 5]])
+    Coordinates:
+        x        (new_dim) <U1 'a' 'b'
+      * y        (y) int64 10 20 30
+      * new_dim  (new_dim) int64 -90 -100
     """
     # TODO: add ignore_index arguments copied from pandas.concat
     # TODO: support concatenating scalar coordinates even if the concatenated
@@ -175,8 +223,7 @@ def concat(
 
     if compat not in _VALID_COMPAT:
         raise ValueError(
-            "compat=%r invalid: must be 'broadcast_equals', 'equals', 'identical', 'no_conflicts' or 'override'"
-            % compat
+            f"compat={compat!r} invalid: must be 'broadcast_equals', 'equals', 'identical', 'no_conflicts' or 'override'"
         )
 
     if isinstance(first_obj, DataArray):
@@ -186,7 +233,7 @@ def concat(
     else:
         raise TypeError(
             "can only concatenate xarray Dataset and DataArray "
-            "objects, got %s" % type(first_obj)
+            f"objects, got {type(first_obj)}"
         )
     return f(
         objs, dim, data_vars, coords, compat, positions, fill_value, join, combine_attrs
@@ -213,6 +260,8 @@ def _calc_concat_dim_coord(dim):
         (dim,) = coord.dims
     else:
         coord = dim
+        if coord.name is None:
+            coord.name = dim.dims[0]
         (dim,) = coord.dims
     return dim, coord
 
@@ -245,18 +294,16 @@ def _calc_concat_over(datasets, dim, dim_names, data_vars, coords, compat):
             if opt == "different":
                 if compat == "override":
                     raise ValueError(
-                        "Cannot specify both %s='different' and compat='override'."
-                        % subset
+                        f"Cannot specify both {subset}='different' and compat='override'."
                     )
                 # all nonindexes that are not the same in each dataset
                 for k in getattr(datasets[0], subset):
                     if k not in concat_over:
                         equals[k] = None
 
-                        variables = []
-                        for ds in datasets:
-                            if k in ds.variables:
-                                variables.append(ds.variables[k])
+                        variables = [
+                            ds.variables[k] for ds in datasets if k in ds.variables
+                        ]
 
                         if len(variables) == 1:
                             # coords="different" doesn't make sense when only one object
@@ -319,12 +366,12 @@ def _calc_concat_over(datasets, dim, dim_names, data_vars, coords, compat):
                 if subset == "coords":
                     raise ValueError(
                         "some variables in coords are not coordinates on "
-                        "the first dataset: %s" % (invalid_vars,)
+                        f"the first dataset: {invalid_vars}"
                     )
                 else:
                     raise ValueError(
                         "some variables in data_vars are not data variables "
-                        "on the first dataset: %s" % (invalid_vars,)
+                        f"on the first dataset: {invalid_vars}"
                     )
             concat_over.update(opt)
 
@@ -335,14 +382,14 @@ def _calc_concat_over(datasets, dim, dim_names, data_vars, coords, compat):
 
 # determine dimensional coordinate names and a dict mapping name to DataArray
 def _parse_datasets(
-    datasets: Iterable["Dataset"],
-) -> Tuple[Dict[Hashable, Variable], Dict[Hashable, int], Set[Hashable], Set[Hashable]]:
+    datasets: Iterable[Dataset],
+) -> tuple[dict[Hashable, Variable], dict[Hashable, int], set[Hashable], set[Hashable]]:
 
-    dims: Set[Hashable] = set()
-    all_coord_names: Set[Hashable] = set()
-    data_vars: Set[Hashable] = set()  # list of data_vars
-    dim_coords: Dict[Hashable, Variable] = {}  # maps dim name to variable
-    dims_sizes: Dict[Hashable, int] = {}  # shared dimension sizes to expand variables
+    dims: set[Hashable] = set()
+    all_coord_names: set[Hashable] = set()
+    data_vars: set[Hashable] = set()  # list of data_vars
+    dim_coords: dict[Hashable, Variable] = {}  # maps dim name to variable
+    dims_sizes: dict[Hashable, int] = {}  # shared dimension sizes to expand variables
 
     for ds in datasets:
         dims_sizes.update(ds.dims)
@@ -362,21 +409,28 @@ def _parse_datasets(
 
 
 def _dataset_concat(
-    datasets: List["Dataset"],
-    dim: Union[str, "DataArray", pd.Index],
-    data_vars: Union[str, List[str]],
-    coords: Union[str, List[str]],
+    datasets: list[Dataset],
+    dim: str | DataArray | pd.Index,
+    data_vars: str | list[str],
+    coords: str | list[str],
     compat: str,
-    positions: Optional[Iterable[int]],
+    positions: Iterable[int] | None,
     fill_value: object = dtypes.NA,
     join: str = "outer",
     combine_attrs: str = "override",
-) -> "Dataset":
+) -> Dataset:
     """
     Concatenate a sequence of datasets along a new or existing dimension
     """
     from .dataset import Dataset
 
+    datasets = list(datasets)
+
+    if not all(isinstance(dataset, Dataset) for dataset in datasets):
+        raise TypeError(
+            "The elements in the input list need to be either all 'Dataset's or all 'DataArray's"
+        )
+
     dim, coord = _calc_concat_dim_coord(dim)
     # Make sure we're working on a copy (we'll be loading variables)
     datasets = [ds.copy() for ds in datasets]
@@ -391,7 +445,7 @@ def _dataset_concat(
     both_data_and_coords = coord_names & data_names
     if both_data_and_coords:
         raise ValueError(
-            "%r is a coordinate in some datasets but not others." % both_data_and_coords
+            f"{both_data_and_coords!r} is a coordinate in some datasets but not others."
         )
     # we don't want the concat dimension in the result dataset yet
     dim_coords.pop(dim, None)
@@ -411,7 +465,7 @@ def _dataset_concat(
 
     result_vars = {}
     if variables_to_merge:
-        to_merge: Dict[Hashable, List[Variable]] = {
+        to_merge: dict[Hashable, list[Variable]] = {
             var: [] for var in variables_to_merge
         }
 
@@ -457,10 +511,10 @@ def _dataset_concat(
     for k in datasets[0].variables:
         if k in concat_over:
             try:
-                vars = ensure_common_dims([ds.variables[k] for ds in datasets])
+                vars = ensure_common_dims([ds[k].variable for ds in datasets])
             except KeyError:
-                raise ValueError("%r is not present in all datasets." % k)
-            combined = concat_vars(vars, dim, positions)
+                raise ValueError(f"{k!r} is not present in all datasets.")
+            combined = concat_vars(vars, dim, positions, combine_attrs=combine_attrs)
             assert isinstance(combined, Variable)
             result_vars[k] = combined
         elif k in result_vars:
@@ -471,8 +525,7 @@ def _dataset_concat(
     absent_coord_names = coord_names - set(result.variables)
     if absent_coord_names:
         raise ValueError(
-            "Variables %r are coordinates in some datasets but not others."
-            % absent_coord_names
+            f"Variables {absent_coord_names!r} are coordinates in some datasets but not others."
         )
     result = result.set_coords(coord_names)
     result.encoding = result_encoding
@@ -487,18 +540,25 @@ def _dataset_concat(
 
 
 def _dataarray_concat(
-    arrays: Iterable["DataArray"],
-    dim: Union[str, "DataArray", pd.Index],
-    data_vars: Union[str, List[str]],
-    coords: Union[str, List[str]],
+    arrays: Iterable[DataArray],
+    dim: str | DataArray | pd.Index,
+    data_vars: str | list[str],
+    coords: str | list[str],
     compat: str,
-    positions: Optional[Iterable[int]],
+    positions: Iterable[int] | None,
     fill_value: object = dtypes.NA,
     join: str = "outer",
     combine_attrs: str = "override",
-) -> "DataArray":
+) -> DataArray:
+    from .dataarray import DataArray
+
     arrays = list(arrays)
 
+    if not all(isinstance(array, DataArray) for array in arrays):
+        raise TypeError(
+            "The elements in the input list need to be either all 'Dataset's or all 'DataArray's"
+        )
+
     if data_vars != "all":
         raise ValueError(
             "data_vars is not a valid argument when concatenating DataArray objects"
@@ -524,7 +584,7 @@ def _dataarray_concat(
         positions,
         fill_value=fill_value,
         join=join,
-        combine_attrs="drop",
+        combine_attrs=combine_attrs,
     )
 
     merged_attrs = merge_attrs([da.attrs for da in arrays], combine_attrs)
