@@ -1,19 +1,19 @@
 import datetime as dt
 import warnings
-from distutils.version import LooseVersion
 from functools import partial
 from numbers import Number
 from typing import Any, Callable, Dict, Hashable, Sequence, Union
 
 import numpy as np
 import pandas as pd
+from packaging.version import Version
 
 from . import utils
 from .common import _contains_datetime_like_objects, ones_like
 from .computation import apply_ufunc
-from .duck_array_ops import datetime_to_numeric, timedelta_to_numeric
-from .options import _get_keep_attrs
-from .pycompat import is_duck_dask_array
+from .duck_array_ops import datetime_to_numeric, push, timedelta_to_numeric
+from .options import OPTIONS, _get_keep_attrs
+from .pycompat import dask_version, is_duck_dask_array
 from .utils import OrderedSet, is_scalar
 from .variable import Variable, broadcast_variables
 
@@ -83,9 +83,11 @@ class NumpyInterpolator(BaseInterpolator):
         self._xi = xi
         self._yi = yi
 
+        nan = np.nan if yi.dtype.kind != "c" else np.nan + np.nan * 1j
+
         if fill_value is None:
-            self._left = np.nan
-            self._right = np.nan
+            self._left = nan
+            self._right = nan
         elif isinstance(fill_value, Sequence) and len(fill_value) == 2:
             self._left = fill_value[0]
             self._right = fill_value[1]
@@ -93,7 +95,7 @@ class NumpyInterpolator(BaseInterpolator):
             self._left = fill_value
             self._right = fill_value
         else:
-            raise ValueError("%s is not a valid fill_value" % fill_value)
+            raise ValueError(f"{fill_value} is not a valid fill_value")
 
     def __call__(self, x):
         return self.f(
@@ -144,17 +146,19 @@ class ScipyInterpolator(BaseInterpolator):
         self.cons_kwargs = kwargs
         self.call_kwargs = {}
 
+        nan = np.nan if yi.dtype.kind != "c" else np.nan + np.nan * 1j
+
         if fill_value is None and method == "linear":
-            fill_value = np.nan, np.nan
+            fill_value = nan, nan
         elif fill_value is None:
-            fill_value = np.nan
+            fill_value = nan
 
         self.f = interp1d(
             xi,
             yi,
             kind=self.method,
             fill_value=fill_value,
-            bounds_error=False,
+            bounds_error=bounds_error,
             assume_sorted=assume_sorted,
             copy=copy,
             **self.cons_kwargs,
@@ -216,20 +220,20 @@ def get_clean_interp_index(
     Parameters
     ----------
     arr : DataArray
-      Array to interpolate or fit to a curve.
+        Array to interpolate or fit to a curve.
     dim : str
-      Name of dimension along which to fit.
+        Name of dimension along which to fit.
     use_coordinate : str or bool
-      If use_coordinate is True, the coordinate that shares the name of the
-      dimension along which interpolation is being performed will be used as the
-      x values. If False, the x values are set as an equally spaced sequence.
+        If use_coordinate is True, the coordinate that shares the name of the
+        dimension along which interpolation is being performed will be used as the
+        x values. If False, the x values are set as an equally spaced sequence.
     strict : bool
-      Whether to raise errors if the index is either non-unique or non-monotonic (default).
+        Whether to raise errors if the index is either non-unique or non-monotonic (default).
 
     Returns
     -------
     Variable
-      Numerical values for the x-coordinates.
+        Numerical values for the x-coordinates.
 
     Notes
     -----
@@ -317,9 +321,13 @@ def interp_na(
         if not is_scalar(max_gap):
             raise ValueError("max_gap must be a scalar.")
 
+        # TODO: benbovy - flexible indexes: update when CFTimeIndex (and DatetimeIndex?)
+        # has its own class inheriting from xarray.Index
         if (
-            dim in self.indexes
-            and isinstance(self.indexes[dim], (pd.DatetimeIndex, CFTimeIndex))
+            dim in self.xindexes
+            and isinstance(
+                self.xindexes[dim].to_pandas_index(), (pd.DatetimeIndex, CFTimeIndex)
+            )
             and use_coordinate
         ):
             # Convert to float
@@ -378,9 +386,9 @@ def func_interpolate_na(interpolator, y, x, **kwargs):
     nans = pd.isnull(y)
     nonans = ~nans
 
-    # fast track for no-nans and all-nans cases
+    # fast track for no-nans, all nan but one, and all-nans cases
     n_nans = nans.sum()
-    if n_nans == 0 or n_nans == len(y):
+    if n_nans == 0 or n_nans >= len(y) - 1:
         return y
 
     f = interpolator(x[nonans], y[nonans], **kwargs)
@@ -390,12 +398,10 @@ def func_interpolate_na(interpolator, y, x, **kwargs):
 
 def _bfill(arr, n=None, axis=-1):
     """inverse of ffill"""
-    import bottleneck as bn
-
     arr = np.flip(arr, axis=axis)
 
     # fill
-    arr = bn.push(arr, axis=axis, n=n)
+    arr = push(arr, axis=axis, n=n)
 
     # reverse back to original
     return np.flip(arr, axis=axis)
@@ -403,7 +409,11 @@ def _bfill(arr, n=None, axis=-1):
 
 def ffill(arr, dim=None, limit=None):
     """forward fill missing values"""
-    import bottleneck as bn
+    if not OPTIONS["use_bottleneck"]:
+        raise RuntimeError(
+            "ffill requires bottleneck to be enabled."
+            " Call `xr.set_options(use_bottleneck=True)` to enable it."
+        )
 
     axis = arr.get_axis_num(dim)
 
@@ -411,9 +421,9 @@ def ffill(arr, dim=None, limit=None):
     _limit = limit if limit is not None else arr.shape[axis]
 
     return apply_ufunc(
-        bn.push,
+        push,
         arr,
-        dask="parallelized",
+        dask="allowed",
         keep_attrs=True,
         output_dtypes=[arr.dtype],
         kwargs=dict(n=_limit, axis=axis),
@@ -422,6 +432,12 @@ def ffill(arr, dim=None, limit=None):
 
 def bfill(arr, dim=None, limit=None):
     """backfill missing values"""
+    if not OPTIONS["use_bottleneck"]:
+        raise RuntimeError(
+            "bfill requires bottleneck to be enabled."
+            " Call `xr.set_options(use_bottleneck=True)` to enable it."
+        )
+
     axis = arr.get_axis_num(dim)
 
     # work around for bottleneck 178
@@ -430,13 +446,23 @@ def bfill(arr, dim=None, limit=None):
     return apply_ufunc(
         _bfill,
         arr,
-        dask="parallelized",
+        dask="allowed",
         keep_attrs=True,
         output_dtypes=[arr.dtype],
         kwargs=dict(n=_limit, axis=axis),
     ).transpose(*arr.dims)
 
 
+def _import_interpolant(interpolant, method):
+    """Import interpolant from scipy.interpolate."""
+    try:
+        from scipy import interpolate
+
+        return getattr(interpolate, interpolant)
+    except ImportError as e:
+        raise ImportError(f"Interpolation with method {method} requires scipy.") from e
+
+
 def _get_interpolator(method, vectorizeable_only=False, **kwargs):
     """helper function to select the appropriate interpolator class
 
@@ -459,12 +485,6 @@ def _get_interpolator(method, vectorizeable_only=False, **kwargs):
         "akima",
     ]
 
-    has_scipy = True
-    try:
-        from scipy import interpolate
-    except ImportError:
-        has_scipy = False
-
     # prioritize scipy.interpolate
     if (
         method == "linear"
@@ -475,32 +495,29 @@ def _get_interpolator(method, vectorizeable_only=False, **kwargs):
         interp_class = NumpyInterpolator
 
     elif method in valid_methods:
-        if not has_scipy:
-            raise ImportError("Interpolation with method `%s` requires scipy" % method)
-
         if method in interp1d_methods:
             kwargs.update(method=method)
             interp_class = ScipyInterpolator
         elif vectorizeable_only:
             raise ValueError(
-                "{} is not a vectorizeable interpolator. "
-                "Available methods are {}".format(method, interp1d_methods)
+                f"{method} is not a vectorizeable interpolator. "
+                f"Available methods are {interp1d_methods}"
             )
         elif method == "barycentric":
-            interp_class = interpolate.BarycentricInterpolator
+            interp_class = _import_interpolant("BarycentricInterpolator", method)
         elif method == "krog":
-            interp_class = interpolate.KroghInterpolator
+            interp_class = _import_interpolant("KroghInterpolator", method)
         elif method == "pchip":
-            interp_class = interpolate.PchipInterpolator
+            interp_class = _import_interpolant("PchipInterpolator", method)
         elif method == "spline":
             kwargs.update(method=method)
             interp_class = SplineInterpolator
         elif method == "akima":
-            interp_class = interpolate.Akima1DInterpolator
+            interp_class = _import_interpolant("Akima1DInterpolator", method)
         else:
-            raise ValueError("%s is not a valid scipy interpolator" % method)
+            raise ValueError(f"{method} is not a valid scipy interpolator")
     else:
-        raise ValueError("%s is not a valid interpolator" % method)
+        raise ValueError(f"{method} is not a valid interpolator")
 
     return interp_class, kwargs
 
@@ -512,18 +529,13 @@ def _get_interpolator_nd(method, **kwargs):
     """
     valid_methods = ["linear", "nearest"]
 
-    try:
-        from scipy import interpolate
-    except ImportError:
-        raise ImportError("Interpolation with method `%s` requires scipy" % method)
-
     if method in valid_methods:
         kwargs.update(method=method)
-        interp_class = interpolate.interpn
+        interp_class = _import_interpolant("interpn", method)
     else:
         raise ValueError(
-            "%s is not a valid interpolator for interpolating "
-            "over multiple dimensions." % method
+            f"{method} is not a valid interpolator for interpolating "
+            "over multiple dimensions."
         )
 
     return interp_class, kwargs
@@ -549,20 +561,11 @@ def _localize(var, indexes_coords):
     """
     indexes = {}
     for dim, [x, new_x] in indexes_coords.items():
-        if np.issubdtype(new_x.dtype, np.datetime64) and LooseVersion(
-            np.__version__
-        ) < LooseVersion("1.18"):
-            # np.nanmin/max changed behaviour for datetime types in numpy 1.18,
-            # see https://github.com/pydata/xarray/pull/3924/files
-            minval = np.min(new_x.values)
-            maxval = np.max(new_x.values)
-        else:
-            minval = np.nanmin(new_x.values)
-            maxval = np.nanmax(new_x.values)
+        minval = np.nanmin(new_x.values)
+        maxval = np.nanmax(new_x.values)
         index = x.to_index()
-        imin = index.get_loc(minval, method="nearest")
-        imax = index.get_loc(maxval, method="nearest")
-
+        imin = index.get_indexer([minval], method="nearest").item()
+        imax = index.get_indexer([maxval], method="nearest").item()
         indexes[dim] = slice(max(imin - 2, 0), imax + 2)
         indexes_coords[dim] = (x[indexes[dim]], new_x)
     return var.isel(**indexes), indexes_coords
@@ -593,16 +596,16 @@ def interp(var, indexes_coords, method, **kwargs):
 
     Parameters
     ----------
-    var: Variable
-    index_coords:
+    var : Variable
+    indexes_coords
         Mapping from dimension name to a pair of original and new coordinates.
         Original coordinates should be sorted in strictly ascending order.
         Note that all the coordinates should be Variable objects.
-    method: string
+    method : string
         One of {'linear', 'nearest', 'zero', 'slinear', 'quadratic',
         'cubic'}. For multidimensional interpolation, only
         {'linear', 'nearest'} can be used.
-    **kwargs:
+    **kwargs
         keyword arguments to be passed to scipy.interpolate
 
     Returns
@@ -625,10 +628,6 @@ def interp(var, indexes_coords, method, **kwargs):
     for indexes_coords in decompose_interp(indexes_coords):
         var = result
 
-        # simple speed up for the local interpolation
-        if method in ["linear", "nearest"]:
-            var, indexes_coords = _localize(var, indexes_coords)
-
         # target dimensions
         dims = list(indexes_coords)
         x, new_x = zip(*[indexes_coords[d] for d in dims])
@@ -651,7 +650,7 @@ def interp(var, indexes_coords, method, **kwargs):
                 out_dims.update(indexes_coords[d][1].dims)
             else:
                 out_dims.add(d)
-        result = result.transpose(*tuple(out_dims))
+        result = result.transpose(*out_dims)
     return result
 
 
@@ -662,17 +661,17 @@ def interp_func(var, x, new_x, method, kwargs):
 
     Parameters
     ----------
-    var: np.ndarray or dask.array.Array
+    var : np.ndarray or dask.array.Array
         Array to be interpolated. The final dimension is interpolated.
-    x: a list of 1d array.
+    x : a list of 1d array.
         Original coordinates. Should not contain NaN.
-    new_x: a list of 1d array
+    new_x : a list of 1d array
         New coordinates. Should not contain NaN.
-    method: string
+    method : string
         {'linear', 'nearest', 'zero', 'slinear', 'quadratic', 'cubic'} for
         1-dimensional interpolation.
         {'linear', 'nearest'} for multidimensional interpolation
-    **kwargs:
+    **kwargs
         Optional keyword arguments to be passed to scipy.interpolator
 
     Returns
@@ -680,8 +679,8 @@ def interp_func(var, x, new_x, method, kwargs):
     interpolated: array
         Interpolated array
 
-    Note
-    ----
+    Notes
+    -----
     This requiers scipy installed.
 
     See Also
@@ -699,33 +698,34 @@ def interp_func(var, x, new_x, method, kwargs):
     if is_duck_dask_array(var):
         import dask.array as da
 
-        nconst = var.ndim - len(x)
+        ndim = var.ndim
+        nconst = ndim - len(x)
 
-        out_ind = list(range(nconst)) + list(range(var.ndim, var.ndim + new_x[0].ndim))
+        out_ind = list(range(nconst)) + list(range(ndim, ndim + new_x[0].ndim))
 
         # blockwise args format
         x_arginds = [[_x, (nconst + index,)] for index, _x in enumerate(x)]
         x_arginds = [item for pair in x_arginds for item in pair]
         new_x_arginds = [
-            [_x, [var.ndim + index for index in range(_x.ndim)]] for _x in new_x
+            [_x, [ndim + index for index in range(_x.ndim)]] for _x in new_x
         ]
         new_x_arginds = [item for pair in new_x_arginds for item in pair]
 
         args = (
             var,
-            range(var.ndim),
+            range(ndim),
             *x_arginds,
             *new_x_arginds,
         )
 
         _, rechunked = da.unify_chunks(*args)
 
-        args = tuple([elem for pair in zip(rechunked, args[1::2]) for elem in pair])
+        args = tuple(elem for pair in zip(rechunked, args[1::2]) for elem in pair)
 
         new_x = rechunked[1 + (len(rechunked) - 1) // 2 :]
 
         new_axes = {
-            var.ndim + i: new_x[0].chunks[i]
+            ndim + i: new_x[0].chunks[i]
             if new_x[0].chunks is not None
             else new_x[0].shape[i]
             for i in range(new_x[0].ndim)
@@ -734,6 +734,20 @@ def interp_func(var, x, new_x, method, kwargs):
         # if usefull, re-use localize for each chunk of new_x
         localize = (method in ["linear", "nearest"]) and (new_x[0].chunks is not None)
 
+        # scipy.interpolate.interp1d always forces to float.
+        # Use the same check for blockwise as well:
+        if not issubclass(var.dtype.type, np.inexact):
+            dtype = np.float_
+        else:
+            dtype = var.dtype
+
+        if dask_version < Version("2020.12"):
+            # Using meta and dtype at the same time doesn't work.
+            # Remove this whenever the minimum requirement for dask is 2020.12:
+            meta = None
+        else:
+            meta = var._meta
+
         return da.blockwise(
             _dask_aware_interpnd,
             out_ind,
@@ -742,8 +756,10 @@ def interp_func(var, x, new_x, method, kwargs):
             interp_kwargs=kwargs,
             localize=localize,
             concatenate=True,
-            dtype=var.dtype,
+            dtype=dtype,
             new_axes=new_axes,
+            meta=meta,
+            align_arrays=False,
         )
 
     return _interpnd(var, x, new_x, func, kwargs)
