@@ -10,7 +10,6 @@ import operator
 import numpy as np
 
 from . import dtypes, duck_array_ops
-from .nputils import array_eq, array_ne
 
 try:
     import bottleneck as bn
@@ -22,8 +21,6 @@ except ImportError:
     has_bottleneck = False
 
 
-UNARY_OPS = ["neg", "pos", "abs", "invert"]
-CMP_BINARY_OPS = ["lt", "le", "ge", "gt"]
 NUM_BINARY_OPS = [
     "add",
     "sub",
@@ -40,9 +37,7 @@ NUM_BINARY_OPS = [
 # methods which pass on the numpy return value unchanged
 # be careful not to list methods that we would want to wrap later
 NUMPY_SAME_METHODS = ["item", "searchsorted"]
-# methods which don't modify the data shape, so the result should still be
-# wrapped in an Variable/DataArray
-NUMPY_UNARY_METHODS = ["argsort", "clip", "conj", "conjugate"]
+
 # methods which remove an axis
 REDUCE_METHODS = ["all", "any"]
 NAN_REDUCE_METHODS = [
@@ -114,23 +109,12 @@ skipna : bool, optional
 
 _MINCOUNT_DOCSTRING = """
 min_count : int, default: None
-    The required number of valid values to perform the operation.
-    If fewer than min_count non-NA values are present the result will
-    be NA. New in version 0.10.8: Added with the default being None."""
-
-_COARSEN_REDUCE_DOCSTRING_TEMPLATE = """\
-Coarsen this object by applying `{name}` along its dimensions.
-
-Parameters
-----------
-**kwargs : dict
-    Additional keyword arguments passed on to `{name}`.
-
-Returns
--------
-reduced : DataArray or Dataset
-    New object with `{name}` applied along its coasen dimnensions.
-"""
+    The required number of valid values to perform the operation. If
+    fewer than min_count non-NA values are present the result will be
+    NA. Only used if skipna is set to True or defaults to True for the
+    array's dtype. New in version 0.10.8: Added with the default being
+    None. Changed in version 0.17.0: if specified on an integer array
+    and skipna=True, the result will be a float array."""
 
 
 def fillna(data, other, join="left", dataset_join="left"):
@@ -252,7 +236,7 @@ def _func_slash_method_wrapper(f, name=None):
 def inject_reduce_methods(cls):
     methods = (
         [
-            (name, getattr(duck_array_ops, "array_%s" % name), False)
+            (name, getattr(duck_array_ops, f"array_{name}"), False)
             for name in REDUCE_METHODS
         ]
         + [(name, getattr(duck_array_ops, name), True) for name in NAN_REDUCE_METHODS]
@@ -291,7 +275,7 @@ def inject_cum_methods(cls):
 
 
 def op_str(name):
-    return "__%s__" % name
+    return f"__{name}__"
 
 
 def get_op(name):
@@ -305,42 +289,44 @@ def inplace_to_noninplace_op(f):
     return NON_INPLACE_OP[f]
 
 
-def inject_binary_ops(cls, inplace=False):
-    for name in CMP_BINARY_OPS + NUM_BINARY_OPS:
-        setattr(cls, op_str(name), cls._binary_op(get_op(name)))
-
-    for name, f in [("eq", array_eq), ("ne", array_ne)]:
-        setattr(cls, op_str(name), cls._binary_op(f))
-
-    for name in NUM_BINARY_OPS:
-        # only numeric operations have in-place and reflexive variants
-        setattr(cls, op_str("r" + name), cls._binary_op(get_op(name), reflexive=True))
-        if inplace:
-            setattr(cls, op_str("i" + name), cls._inplace_binary_op(get_op("i" + name)))
+# _typed_ops.py uses the following wrapped functions as a kind of unary operator
+argsort = _method_wrapper("argsort")
+conj = _method_wrapper("conj")
+conjugate = _method_wrapper("conjugate")
+round_ = _func_slash_method_wrapper(duck_array_ops.around, name="round")
 
 
-def inject_all_ops_and_reduce_methods(cls, priority=50, array_only=True):
-    # prioritize our operations over those of numpy.ndarray (priority=1)
-    # and numpy.matrix (priority=10)
-    cls.__array_priority__ = priority
+def inject_numpy_same(cls):
+    # these methods don't return arrays of the same shape as the input, so
+    # don't try to patch these in for Dataset objects
+    for name in NUMPY_SAME_METHODS:
+        setattr(cls, name, _values_method_wrapper(name))
 
-    # patch in standard special operations
-    for name in UNARY_OPS:
-        setattr(cls, op_str(name), cls._unary_op(get_op(name)))
-    inject_binary_ops(cls, inplace=True)
 
-    # patch in numpy/pandas methods
-    for name in NUMPY_UNARY_METHODS:
-        setattr(cls, name, cls._unary_op(_method_wrapper(name)))
+class IncludeReduceMethods:
+    __slots__ = ()
 
-    f = _func_slash_method_wrapper(duck_array_ops.around, name="round")
-    setattr(cls, "round", cls._unary_op(f))
+    def __init_subclass__(cls, **kwargs):
+        super().__init_subclass__(**kwargs)
 
-    if array_only:
-        # these methods don't return arrays of the same shape as the input, so
-        # don't try to patch these in for Dataset objects
-        for name in NUMPY_SAME_METHODS:
-            setattr(cls, name, _values_method_wrapper(name))
+        if getattr(cls, "_reduce_method", None):
+            inject_reduce_methods(cls)
 
-    inject_reduce_methods(cls)
-    inject_cum_methods(cls)
+
+class IncludeCumMethods:
+    __slots__ = ()
+
+    def __init_subclass__(cls, **kwargs):
+        super().__init_subclass__(**kwargs)
+
+        if getattr(cls, "_reduce_method", None):
+            inject_cum_methods(cls)
+
+
+class IncludeNumpySameMethods:
+    __slots__ = ()
+
+    def __init_subclass__(cls, **kwargs):
+        super().__init_subclass__(**kwargs)
+
+        inject_numpy_same(cls)  # some methods not applicable to Dataset objects
