@@ -6,12 +6,16 @@ import numpy as np
 
 from ..core.formatting import format_item
 from .utils import (
-    _infer_xy_labels, _process_cmap_cbar_kwargs, import_matplotlib_pyplot,
-    label_from_attrs)
+    _get_nice_quiver_magnitude,
+    _infer_xy_labels,
+    _process_cmap_cbar_kwargs,
+    import_matplotlib_pyplot,
+    label_from_attrs,
+)
 
 # Overrides axes.labelsize, xtick.major.size, ytick.major.size
 # from mpl.rcParams
-_FONTSIZE = 'small'
+_FONTSIZE = "small"
 # For major ticks on x, y axes
 _NTICKS = 5
 
@@ -24,20 +28,20 @@ def _nicetitle(coord, value, maxchar, template):
     title = template.format(coord=coord, value=prettyvalue)
 
     if len(title) > maxchar:
-        title = title[:(maxchar - 3)] + '...'
+        title = title[: (maxchar - 3)] + "..."
 
     return title
 
 
 class FacetGrid:
     """
-    Initialize the matplotlib figure and FacetGrid object.
+    Initialize the Matplotlib figure and FacetGrid object.
 
     The :class:`FacetGrid` is an object that links a xarray DataArray to
-    a matplotlib figure with a particular structure.
+    a Matplotlib figure with a particular structure.
 
     In particular, :class:`FacetGrid` is used to draw plots with multiple
-    Axes where each Axes shows the same relationship conditioned on
+    axes, where each axes shows the same relationship conditioned on
     different levels of some dimension. It's possible to condition on up to
     two variables by assigning variables to the rows and columns of the
     grid.
@@ -55,45 +59,60 @@ class FacetGrid:
 
     Attributes
     ----------
-    axes : numpy object array
-        Contains axes in corresponding position, as returned from
-        plt.subplots
-    fig : matplotlib.Figure
-        The figure containing all the axes
-    name_dicts : numpy object array
-        Contains dictionaries mapping coordinate names to values. None is
-        used as a sentinel value for axes which should remain empty, ie.
-        sometimes the bottom right grid
-
+    axes : ndarray of matplotlib.axes.Axes
+        Array containing axes in corresponding position, as returned from
+        :py:func:`matplotlib.pyplot.subplots`.
+    col_labels : list of matplotlib.text.Text
+        Column titles.
+    row_labels : list of matplotlib.text.Text
+        Row titles.
+    fig : matplotlib.figure.Figure
+        The figure containing all the axes.
+    name_dicts : ndarray of dict
+        Array containing dictionaries mapping coordinate names to values. ``None`` is
+        used as a sentinel value for axes that should remain empty, i.e.,
+        sometimes the rightmost grid positions in the bottom row.
     """
 
-    def __init__(self, data, col=None, row=None, col_wrap=None,
-                 sharex=True, sharey=True, figsize=None, aspect=1, size=3,
-                 subplot_kws=None):
+    def __init__(
+        self,
+        data,
+        col=None,
+        row=None,
+        col_wrap=None,
+        sharex=True,
+        sharey=True,
+        figsize=None,
+        aspect=1,
+        size=3,
+        subplot_kws=None,
+    ):
         """
         Parameters
         ----------
         data : DataArray
-            xarray DataArray to be plotted
-        row, col : strings
-            Dimesion names that define subsets of the data, which will be drawn
+            xarray DataArray to be plotted.
+        row, col : str
+            Dimension names that define subsets of the data, which will be drawn
             on separate facets in the grid.
         col_wrap : int, optional
-            "Wrap" the column variable at this width, so that the column facets
+            "Wrap" the grid the for the column variable after this number of columns,
+            adding rows if ``col_wrap`` is less than the number of facets.
         sharex : bool, optional
-            If true, the facets will share x axes
+            If true, the facets will share *x* axes.
         sharey : bool, optional
-            If true, the facets will share y axes
+            If true, the facets will share *y* axes.
         figsize : tuple, optional
             A tuple (width, height) of the figure in inches.
             If set, overrides ``size`` and ``aspect``.
         aspect : scalar, optional
             Aspect ratio of each facet, so that ``aspect * size`` gives the
-            width of each facet in inches
+            width of each facet in inches.
         size : scalar, optional
-            Height (in inches) of each facet. See also: ``aspect``
+            Height (in inches) of each facet. See also: ``aspect``.
         subplot_kws : dict, optional
-            Dictionary of keyword arguments for matplotlib subplots
+            Dictionary of keyword arguments for Matplotlib subplots
+            (:py:func:`matplotlib.pyplot.subplots`).
 
         """
 
@@ -103,8 +122,10 @@ class FacetGrid:
         rep_col = col is not None and not data[col].to_index().is_unique
         rep_row = row is not None and not data[row].to_index().is_unique
         if rep_col or rep_row:
-            raise ValueError('Coordinates used for faceting cannot '
-                             'contain repeated (nonunique) values.')
+            raise ValueError(
+                "Coordinates used for faceting cannot "
+                "contain repeated (nonunique) values."
+            )
 
         # single_group is the grouping variable, if there is exactly one
         if col and row:
@@ -113,15 +134,13 @@ class FacetGrid:
             ncol = len(data[col])
             nfacet = nrow * ncol
             if col_wrap is not None:
-                warnings.warn('Ignoring col_wrap since both col and row '
-                              'were passed')
+                warnings.warn("Ignoring col_wrap since both col and row were passed")
         elif row and not col:
             single_group = row
         elif not row and col:
             single_group = col
         else:
-            raise ValueError(
-                'Pass a coordinate name as an argument for row or col')
+            raise ValueError("Pass a coordinate name as an argument for row or col")
 
         # Compute grid shape
         if single_group:
@@ -145,17 +164,22 @@ class FacetGrid:
             cbar_space = 1
             figsize = (ncol * size * aspect + cbar_space, nrow * size)
 
-        fig, axes = plt.subplots(nrow, ncol,
-                                 sharex=sharex, sharey=sharey, squeeze=False,
-                                 figsize=figsize, subplot_kw=subplot_kws)
+        fig, axes = plt.subplots(
+            nrow,
+            ncol,
+            sharex=sharex,
+            sharey=sharey,
+            squeeze=False,
+            figsize=figsize,
+            subplot_kw=subplot_kws,
+        )
 
         # Set up the lists of names for the row and column facet variables
-        col_names = list(data[col].values) if col else []
-        row_names = list(data[row].values) if row else []
+        col_names = list(data[col].to_numpy()) if col else []
+        row_names = list(data[row].to_numpy()) if row else []
 
         if single_group:
-            full = [{single_group: x} for x in
-                    data[single_group].values]
+            full = [{single_group: x} for x in data[single_group].to_numpy()]
             empty = [None for x in range(nrow * ncol - len(full))]
             name_dicts = full + empty
         else:
@@ -175,6 +199,11 @@ class FacetGrid:
         self.row_names = row_names
         self.col_names = col_names
 
+        # guides
+        self.figlegend = None
+        self.quiverkey = None
+        self.cbar = None
+
         # Next the private variables
         self._single_group = single_group
         self._nrow = nrow
@@ -182,6 +211,8 @@ class FacetGrid:
         self._ncol = ncol
         self._col_var = col
         self._col_wrap = col_wrap
+        self.row_labels = [None] * nrow
+        self.col_labels = [None] * ncol
         self._x_var = None
         self._y_var = None
         self._cmap_extend = None
@@ -209,7 +240,7 @@ class FacetGrid:
             plotting method such as `xarray.plot.imshow`
         x, y : string
             Names of the coordinates to plot on x, y axes
-        kwargs :
+        **kwargs
             additional keyword arguments to func
 
         Returns
@@ -218,57 +249,77 @@ class FacetGrid:
 
         """
 
-        if kwargs.get('cbar_ax', None) is not None:
-            raise ValueError('cbar_ax not supported by FacetGrid.')
+        if kwargs.get("cbar_ax", None) is not None:
+            raise ValueError("cbar_ax not supported by FacetGrid.")
 
         cmap_params, cbar_kwargs = _process_cmap_cbar_kwargs(
-            func, self.data.values, **kwargs)
+            func, self.data.to_numpy(), **kwargs
+        )
 
-        self._cmap_extend = cmap_params.get('extend')
+        self._cmap_extend = cmap_params.get("extend")
 
         # Order is important
         func_kwargs = {
-            k: v for k, v in kwargs.items()
-            if k not in {'cmap', 'colors', 'cbar_kwargs', 'levels'}
+            k: v
+            for k, v in kwargs.items()
+            if k not in {"cmap", "colors", "cbar_kwargs", "levels"}
         }
         func_kwargs.update(cmap_params)
-        func_kwargs.update({'add_colorbar': False, 'add_labels': False})
+        func_kwargs["add_colorbar"] = False
+        if func.__name__ != "surface":
+            func_kwargs["add_labels"] = False
 
         # Get x, y labels for the first subplot
         x, y = _infer_xy_labels(
-            darray=self.data.loc[self.name_dicts.flat[0]], x=x, y=y,
-            imshow=func.__name__ == 'imshow', rgb=kwargs.get('rgb', None))
+            darray=self.data.loc[self.name_dicts.flat[0]],
+            x=x,
+            y=y,
+            imshow=func.__name__ == "imshow",
+            rgb=kwargs.get("rgb", None),
+        )
 
         for d, ax in zip(self.name_dicts.flat, self.axes.flat):
             # None is the sentinel value
             if d is not None:
                 subset = self.data.loc[d]
-                mappable = func(subset, x=x, y=y, ax=ax, **func_kwargs)
+                mappable = func(
+                    subset, x=x, y=y, ax=ax, **func_kwargs, _is_facetgrid=True
+                )
                 self._mappables.append(mappable)
 
-        self._cmap_extend = cmap_params.get('extend')
         self._finalize_grid(x, y)
 
-        if kwargs.get('add_colorbar', True):
+        if kwargs.get("add_colorbar", True):
             self.add_colorbar(**cbar_kwargs)
 
         return self
 
-    def map_dataarray_line(self, func, x, y, hue, add_legend=True,
-                           _labels=None, **kwargs):
+    def map_dataarray_line(
+        self, func, x, y, hue, add_legend=True, _labels=None, **kwargs
+    ):
         from .plot import _infer_line_data
 
         for d, ax in zip(self.name_dicts.flat, self.axes.flat):
             # None is the sentinel value
             if d is not None:
                 subset = self.data.loc[d]
-                mappable = func(subset, x=x, y=y, ax=ax,
-                                hue=hue, add_legend=False, _labels=False,
-                                **kwargs)
+                mappable = func(
+                    subset,
+                    x=x,
+                    y=y,
+                    ax=ax,
+                    hue=hue,
+                    add_legend=False,
+                    _labels=False,
+                    **kwargs,
+                )
                 self._mappables.append(mappable)
 
-        _, _, hueplt, xlabel, ylabel, huelabel = _infer_line_data(
-            darray=self.data.loc[self.name_dicts.flat[0]], x=x, y=y, hue=hue)
+        xplt, yplt, hueplt, huelabel = _infer_line_data(
+            darray=self.data.loc[self.name_dicts.flat[0]], x=x, y=y, hue=hue
+        )
+        xlabel = label_from_attrs(xplt)
+        ylabel = label_from_attrs(yplt)
 
         self._hue_var = hueplt
         self._hue_label = huelabel
@@ -279,6 +330,62 @@ class FacetGrid:
 
         return self
 
+    def map_dataset(
+        self, func, x=None, y=None, hue=None, hue_style=None, add_guide=None, **kwargs
+    ):
+        from .dataset_plot import _infer_meta_data, _parse_size
+
+        kwargs["add_guide"] = False
+
+        if kwargs.get("markersize", None):
+            kwargs["size_mapping"] = _parse_size(
+                self.data[kwargs["markersize"]], kwargs.pop("size_norm", None)
+            )
+
+        meta_data = _infer_meta_data(
+            self.data, x, y, hue, hue_style, add_guide, funcname=func.__name__
+        )
+        kwargs["meta_data"] = meta_data
+
+        if hue and meta_data["hue_style"] == "continuous":
+            cmap_params, cbar_kwargs = _process_cmap_cbar_kwargs(
+                func, self.data[hue].to_numpy(), **kwargs
+            )
+            kwargs["meta_data"]["cmap_params"] = cmap_params
+            kwargs["meta_data"]["cbar_kwargs"] = cbar_kwargs
+
+        kwargs["_is_facetgrid"] = True
+
+        if func.__name__ == "quiver" and "scale" not in kwargs:
+            raise ValueError("Please provide scale.")
+            # TODO: come up with an algorithm for reasonable scale choice
+
+        for d, ax in zip(self.name_dicts.flat, self.axes.flat):
+            # None is the sentinel value
+            if d is not None:
+                subset = self.data.loc[d]
+                maybe_mappable = func(
+                    ds=subset, x=x, y=y, hue=hue, hue_style=hue_style, ax=ax, **kwargs
+                )
+                # TODO: this is needed to get legends to work.
+                # but maybe_mappable is a list in that case :/
+                self._mappables.append(maybe_mappable)
+
+        self._finalize_grid(meta_data["xlabel"], meta_data["ylabel"])
+
+        if hue:
+            self._hue_label = meta_data.pop("hue_label", None)
+            if meta_data["add_legend"]:
+                self._hue_var = meta_data["hue"]
+                self.add_legend()
+            elif meta_data["add_colorbar"]:
+                self.add_colorbar(label=self._hue_label, **cbar_kwargs)
+
+        if meta_data["add_quiverkey"]:
+            self.add_quiverkey(kwargs["u"], kwargs["v"])
+
+        return self
+
     def _finalize_grid(self, *axlabels):
         """Finalize the annotations and layout."""
         if not self._finalized:
@@ -292,27 +399,22 @@ class FacetGrid:
 
             self._finalized = True
 
-    def add_legend(self, **kwargs):
-        figlegend = self.fig.legend(
-            handles=self._mappables[-1],
-            labels=list(self._hue_var.values),
-            title=self._hue_label,
-            loc="center right", **kwargs)
-
+    def _adjust_fig_for_guide(self, guide):
         # Draw the plot to set the bounding boxes correctly
-        self.fig.draw(self.fig.canvas.get_renderer())
+        renderer = self.fig.canvas.get_renderer()
+        self.fig.draw(renderer)
 
         # Calculate and set the new width of the figure so the legend fits
-        legend_width = figlegend.get_window_extent().width / self.fig.dpi
+        guide_width = guide.get_window_extent(renderer).width / self.fig.dpi
         figure_width = self.fig.get_figwidth()
-        self.fig.set_figwidth(figure_width + legend_width)
+        self.fig.set_figwidth(figure_width + guide_width)
 
         # Draw the plot again to get the new transformations
-        self.fig.draw(self.fig.canvas.get_renderer())
+        self.fig.draw(renderer)
 
         # Now calculate how much space we need on the right side
-        legend_width = figlegend.get_window_extent().width / self.fig.dpi
-        space_needed = legend_width / (figure_width + legend_width) + 0.02
+        guide_width = guide.get_window_extent(renderer).width / self.fig.dpi
+        space_needed = guide_width / (figure_width + guide_width) + 0.02
         # margin = .01
         # _space_needed = margin + space_needed
         right = 1 - space_needed
@@ -320,17 +422,49 @@ class FacetGrid:
         # Place the subplot axes to give space for the legend
         self.fig.subplots_adjust(right=right)
 
+    def add_legend(self, **kwargs):
+        self.figlegend = self.fig.legend(
+            handles=self._mappables[-1],
+            labels=list(self._hue_var.to_numpy()),
+            title=self._hue_label,
+            loc="center right",
+            **kwargs,
+        )
+        self._adjust_fig_for_guide(self.figlegend)
+
     def add_colorbar(self, **kwargs):
-        """Draw a colorbar
-        """
+        """Draw a colorbar."""
         kwargs = kwargs.copy()
         if self._cmap_extend is not None:
-            kwargs.setdefault('extend', self._cmap_extend)
-        if 'label' not in kwargs:
-            kwargs.setdefault('label', label_from_attrs(self.data))
-        self.cbar = self.fig.colorbar(self._mappables[-1],
-                                      ax=list(self.axes.flat),
-                                      **kwargs)
+            kwargs.setdefault("extend", self._cmap_extend)
+        # dont pass extend as kwarg if it is in the mappable
+        if hasattr(self._mappables[-1], "extend"):
+            kwargs.pop("extend", None)
+        if "label" not in kwargs:
+            kwargs.setdefault("label", label_from_attrs(self.data))
+        self.cbar = self.fig.colorbar(
+            self._mappables[-1], ax=list(self.axes.flat), **kwargs
+        )
+        return self
+
+    def add_quiverkey(self, u, v, **kwargs):
+        kwargs = kwargs.copy()
+
+        magnitude = _get_nice_quiver_magnitude(self.data[u], self.data[v])
+        units = self.data[u].attrs.get("units", "")
+        self.quiverkey = self.axes.flat[-1].quiverkey(
+            self._mappables[-1],
+            X=0.8,
+            Y=0.9,
+            U=magnitude,
+            label=f"{magnitude}\n{units}",
+            labelpos="E",
+            coordinates="figure",
+        )
+
+        # TODO: does not work because self.quiverkey.get_window_extent(renderer) = 0
+        # https://github.com/matplotlib/matplotlib/issues/18530
+        # self._adjust_fig_for_guide(self.quiverkey.text)
         return self
 
     def set_axis_labels(self, x_var=None, y_var=None):
@@ -367,8 +501,7 @@ class FacetGrid:
             ax.set_ylabel(label, **kwargs)
         return self
 
-    def set_titles(self, template="{coord} = {value}", maxchar=30,
-                   size=None, **kwargs):
+    def set_titles(self, template="{coord} = {value}", maxchar=30, size=None, **kwargs):
         """
         Draw titles either above each facet or on the grid margins.
 
@@ -378,7 +511,7 @@ class FacetGrid:
             Template for plot titles containing {coord} and {value}
         maxchar : int
             Truncate titles at maxchar
-        kwargs : keyword args
+        **kwargs : keyword args
             additional arguments to matplotlib.text
 
         Returns
@@ -391,8 +524,7 @@ class FacetGrid:
         if size is None:
             size = mpl.rcParams["axes.labelsize"]
 
-        nicetitle = functools.partial(_nicetitle, maxchar=maxchar,
-                                      template=template)
+        nicetitle = functools.partial(_nicetitle, maxchar=maxchar, template=template)
 
         if self._single_group:
             for d, ax in zip(self.name_dicts.flat, self.axes.flat):
@@ -403,24 +535,38 @@ class FacetGrid:
                     ax.set_title(title, size=size, **kwargs)
         else:
             # The row titles on the right edge of the grid
-            for ax, row_name in zip(self.axes[:, -1], self.row_names):
-                title = nicetitle(coord=self._row_var, value=row_name,
-                                  maxchar=maxchar)
-                ax.annotate(title, xy=(1.02, .5), xycoords="axes fraction",
-                            rotation=270, ha="left", va="center", **kwargs)
+            for index, (ax, row_name, handle) in enumerate(
+                zip(self.axes[:, -1], self.row_names, self.row_labels)
+            ):
+                title = nicetitle(coord=self._row_var, value=row_name, maxchar=maxchar)
+                if not handle:
+                    self.row_labels[index] = ax.annotate(
+                        title,
+                        xy=(1.02, 0.5),
+                        xycoords="axes fraction",
+                        rotation=270,
+                        ha="left",
+                        va="center",
+                        **kwargs,
+                    )
+                else:
+                    handle.set_text(title)
 
             # The column titles on the top row
-            for ax, col_name in zip(self.axes[0, :], self.col_names):
-                title = nicetitle(coord=self._col_var, value=col_name,
-                                  maxchar=maxchar)
-                ax.set_title(title, size=size, **kwargs)
+            for index, (ax, col_name, handle) in enumerate(
+                zip(self.axes[0, :], self.col_names, self.col_labels)
+            ):
+                title = nicetitle(coord=self._col_var, value=col_name, maxchar=maxchar)
+                if not handle:
+                    self.col_labels[index] = ax.set_title(title, size=size, **kwargs)
+                else:
+                    handle.set_text(title)
 
         return self
 
-    def set_ticks(self, max_xticks=_NTICKS, max_yticks=_NTICKS,
-                  fontsize=_FONTSIZE):
+    def set_ticks(self, max_xticks=_NTICKS, max_yticks=_NTICKS, fontsize=_FONTSIZE):
         """
-        Set and control tick behavior
+        Set and control tick behavior.
 
         Parameters
         ----------
@@ -443,8 +589,9 @@ class FacetGrid:
         for ax in self.axes.flat:
             ax.xaxis.set_major_locator(x_major_locator)
             ax.yaxis.set_major_locator(y_major_locator)
-            for tick in itertools.chain(ax.xaxis.get_major_ticks(),
-                                        ax.yaxis.get_major_ticks()):
+            for tick in itertools.chain(
+                ax.xaxis.get_major_ticks(), ax.yaxis.get_major_ticks()
+            ):
                 tick.label1.set_fontsize(fontsize)
 
         return self
@@ -460,11 +607,11 @@ class FacetGrid:
             must plot to the currently active matplotlib Axes and take a
             `color` keyword argument. If faceting on the `hue` dimension,
             it must also take a `label` keyword argument.
-        args : strings
+        *args : strings
             Column names in self.data that identify variables with data to
             plot. The data for each variable is passed to `func` in the
             order the variables are specified in the call.
-        kwargs : keyword arguments
+        **kwargs : keyword arguments
             All keyword arguments are passed to the plotting function.
 
         Returns
@@ -478,12 +625,11 @@ class FacetGrid:
             if namedict is not None:
                 data = self.data.loc[namedict]
                 plt.sca(ax)
-                innerargs = [data[a].values for a in args]
+                innerargs = [data[a].to_numpy() for a in args]
                 maybe_mappable = func(*innerargs, **kwargs)
                 # TODO: better way to verify that an artist is mappable?
                 # https://stackoverflow.com/questions/33023036/is-it-possible-to-detect-if-a-matplotlib-artist-is-a-mappable-suitable-for-use-w#33023522
-                if (maybe_mappable and
-                        hasattr(maybe_mappable, 'autoscale_None')):
+                if maybe_mappable and hasattr(maybe_mappable, "autoscale_None"):
                     self._mappables.append(maybe_mappable)
 
         self._finalize_grid(*args[:2])
@@ -491,10 +637,24 @@ class FacetGrid:
         return self
 
 
-def _easy_facetgrid(data, plotfunc, kind, x=None, y=None, row=None,
-                    col=None, col_wrap=None, sharex=True, sharey=True,
-                    aspect=None, size=None, subplot_kws=None, ax=None,
-                    figsize=None, **kwargs):
+def _easy_facetgrid(
+    data,
+    plotfunc,
+    kind,
+    x=None,
+    y=None,
+    row=None,
+    col=None,
+    col_wrap=None,
+    sharex=True,
+    sharey=True,
+    aspect=None,
+    size=None,
+    subplot_kws=None,
+    ax=None,
+    figsize=None,
+    **kwargs,
+):
     """
     Convenience method to call xarray.plot.FacetGrid from 2d plotting methods
 
@@ -507,14 +667,26 @@ def _easy_facetgrid(data, plotfunc, kind, x=None, y=None, row=None,
     if size is None:
         size = 3
     elif figsize is not None:
-        raise ValueError('cannot provide both `figsize` and `size` arguments')
+        raise ValueError("cannot provide both `figsize` and `size` arguments")
 
-    g = FacetGrid(data=data, col=col, row=row, col_wrap=col_wrap,
-                  sharex=sharex, sharey=sharey, figsize=figsize,
-                  aspect=aspect, size=size, subplot_kws=subplot_kws)
+    g = FacetGrid(
+        data=data,
+        col=col,
+        row=row,
+        col_wrap=col_wrap,
+        sharex=sharex,
+        sharey=sharey,
+        figsize=figsize,
+        aspect=aspect,
+        size=size,
+        subplot_kws=subplot_kws,
+    )
 
-    if kind == 'line':
+    if kind == "line":
         return g.map_dataarray_line(plotfunc, x, y, **kwargs)
 
-    if kind == 'dataarray':
+    if kind == "dataarray":
         return g.map_dataarray(plotfunc, x, y, **kwargs)
+
+    if kind == "dataset":
+        return g.map_dataset(plotfunc, x, y, **kwargs)
