@@ -1,5 +1,3 @@
-from distutils.version import LooseVersion
-
 import numpy as np
 import pandas as pd
 import pytest
@@ -12,7 +10,6 @@ from . import (
     assert_equal,
     assert_identical,
     raise_if_dask_computes,
-    raises_regex,
     requires_cftime,
     requires_dask,
 )
@@ -59,6 +56,8 @@ class TestDatetimeAccessor:
             "weekday",
             "dayofyear",
             "quarter",
+            "date",
+            "time",
             "is_month_start",
             "is_month_end",
             "is_quarter_start",
@@ -68,9 +67,9 @@ class TestDatetimeAccessor:
             "is_leap_year",
         ],
     )
-    def test_field_access(self, field):
+    def test_field_access(self, field) -> None:
 
-        if LooseVersion(pd.__version__) >= "1.1.0" and field in ["week", "weekofyear"]:
+        if field in ["week", "weekofyear"]:
             data = self.times.isocalendar()["week"]
         else:
             data = getattr(self.times, field)
@@ -95,14 +94,7 @@ class TestDatetimeAccessor:
             ("weekday", "day"),
         ],
     )
-    def test_isocalendar(self, field, pandas_field):
-
-        if LooseVersion(pd.__version__) < "1.1.0":
-            with raises_regex(
-                AttributeError, "'isocalendar' not available in pandas < 1.1.0"
-            ):
-                self.data.time.dt.isocalendar()[field]
-            return
+    def test_isocalendar(self, field, pandas_field) -> None:
 
         # pandas isocalendar has dtypy UInt32Dtype, convert to Int64
         expected = pd.Int64Index(getattr(self.times.isocalendar(), pandas_field))
@@ -113,16 +105,20 @@ class TestDatetimeAccessor:
         actual = self.data.time.dt.isocalendar()[field]
         assert_equal(expected, actual)
 
-    def test_strftime(self):
+    def test_calendar(self) -> None:
+        cal = self.data.time.dt.calendar
+        assert cal == "proleptic_gregorian"
+
+    def test_strftime(self) -> None:
         assert (
             "2000-01-01 01:00:00" == self.data.time.dt.strftime("%Y-%m-%d %H:%M:%S")[1]
         )
 
-    def test_not_datetime_type(self):
+    def test_not_datetime_type(self) -> None:
         nontime_data = self.data.copy()
         int_data = np.arange(len(self.data.time)).astype("int8")
         nontime_data = nontime_data.assign_coords(time=int_data)
-        with raises_regex(TypeError, "dt"):
+        with pytest.raises(TypeError, match=r"dt"):
             nontime_data.time.dt
 
     @pytest.mark.filterwarnings("ignore:dt.weekofyear and dt.week have been deprecated")
@@ -144,6 +140,8 @@ class TestDatetimeAccessor:
             "weekday",
             "dayofyear",
             "quarter",
+            "date",
+            "time",
             "is_month_start",
             "is_month_end",
             "is_quarter_start",
@@ -153,7 +151,7 @@ class TestDatetimeAccessor:
             "is_leap_year",
         ],
     )
-    def test_dask_field_access(self, field):
+    def test_dask_field_access(self, field) -> None:
         import dask.array as da
 
         expected = getattr(self.times_data.dt, field)
@@ -179,16 +177,9 @@ class TestDatetimeAccessor:
             "weekday",
         ],
     )
-    def test_isocalendar_dask(self, field):
+    def test_isocalendar_dask(self, field) -> None:
         import dask.array as da
 
-        if LooseVersion(pd.__version__) < "1.1.0":
-            with raises_regex(
-                AttributeError, "'isocalendar' not available in pandas < 1.1.0"
-            ):
-                self.data.time.dt.isocalendar()[field]
-            return
-
         expected = getattr(self.times_data.dt.isocalendar(), field)
 
         dask_times_arr = da.from_array(self.times_arr, chunks=(5, 5, 50))
@@ -213,7 +204,7 @@ class TestDatetimeAccessor:
             ("strftime", "%Y-%m-%d %H:%M:%S"),
         ],
     )
-    def test_dask_accessor_method(self, method, parameters):
+    def test_dask_accessor_method(self, method, parameters) -> None:
         import dask.array as da
 
         expected = getattr(self.times_data.dt, method)(parameters)
@@ -229,31 +220,34 @@ class TestDatetimeAccessor:
         assert_chunks_equal(actual, dask_times_2d)
         assert_equal(actual.compute(), expected.compute())
 
-    def test_seasons(self):
+    def test_seasons(self) -> None:
         dates = pd.date_range(start="2000/01/01", freq="M", periods=12)
+        dates = dates.append(pd.Index([np.datetime64("NaT")]))
         dates = xr.DataArray(dates)
-        seasons = [
-            "DJF",
-            "DJF",
-            "MAM",
-            "MAM",
-            "MAM",
-            "JJA",
-            "JJA",
-            "JJA",
-            "SON",
-            "SON",
-            "SON",
-            "DJF",
-        ]
-        seasons = xr.DataArray(seasons)
+        seasons = xr.DataArray(
+            [
+                "DJF",
+                "DJF",
+                "MAM",
+                "MAM",
+                "MAM",
+                "JJA",
+                "JJA",
+                "JJA",
+                "SON",
+                "SON",
+                "SON",
+                "DJF",
+                "nan",
+            ]
+        )
 
         assert_array_equal(seasons.values, dates.dt.season.values)
 
     @pytest.mark.parametrize(
         "method, parameters", [("floor", "D"), ("ceil", "D"), ("round", "D")]
     )
-    def test_accessor_method(self, method, parameters):
+    def test_accessor_method(self, method, parameters) -> None:
         dates = pd.date_range("2014-01-01", "2014-05-01", freq="H")
         xdates = xr.DataArray(dates, dims=["time"])
         expected = getattr(dates, method)(parameters)
@@ -285,17 +279,17 @@ class TestTimedeltaAccessor:
             name="data",
         )
 
-    def test_not_datetime_type(self):
+    def test_not_datetime_type(self) -> None:
         nontime_data = self.data.copy()
         int_data = np.arange(len(self.data.time)).astype("int8")
         nontime_data = nontime_data.assign_coords(time=int_data)
-        with raises_regex(TypeError, "dt"):
+        with pytest.raises(TypeError, match=r"dt"):
             nontime_data.time.dt
 
     @pytest.mark.parametrize(
         "field", ["days", "seconds", "microseconds", "nanoseconds"]
     )
-    def test_field_access(self, field):
+    def test_field_access(self, field) -> None:
         expected = xr.DataArray(
             getattr(self.times, field), name=field, coords=[self.times], dims=["time"]
         )
@@ -305,7 +299,7 @@ class TestTimedeltaAccessor:
     @pytest.mark.parametrize(
         "method, parameters", [("floor", "D"), ("ceil", "D"), ("round", "D")]
     )
-    def test_accessor_methods(self, method, parameters):
+    def test_accessor_methods(self, method, parameters) -> None:
         dates = pd.timedelta_range(start="1 day", end="30 days", freq="6H")
         xdates = xr.DataArray(dates, dims=["time"])
         expected = getattr(dates, method)(parameters)
@@ -316,7 +310,7 @@ class TestTimedeltaAccessor:
     @pytest.mark.parametrize(
         "field", ["days", "seconds", "microseconds", "nanoseconds"]
     )
-    def test_dask_field_access(self, field):
+    def test_dask_field_access(self, field) -> None:
         import dask.array as da
 
         expected = getattr(self.times_data.dt, field)
@@ -337,7 +331,7 @@ class TestTimedeltaAccessor:
     @pytest.mark.parametrize(
         "method, parameters", [("floor", "D"), ("ceil", "D"), ("round", "D")]
     )
-    def test_dask_accessor_method(self, method, parameters):
+    def test_dask_accessor_method(self, method, parameters) -> None:
         import dask.array as da
 
         expected = getattr(self.times_data.dt, method)(parameters)
@@ -407,9 +401,8 @@ def times_3d(times):
 @pytest.mark.parametrize(
     "field", ["year", "month", "day", "hour", "dayofyear", "dayofweek"]
 )
-def test_field_access(data, field):
-    if field == "dayofyear" or field == "dayofweek":
-        pytest.importorskip("cftime", minversion="1.0.2.1")
+def test_field_access(data, field) -> None:
+
     result = getattr(data.time.dt, field)
     expected = xr.DataArray(
         getattr(xr.coding.cftimeindex.CFTimeIndex(data.time.values), field),
@@ -422,18 +415,74 @@ def test_field_access(data, field):
 
 
 @requires_cftime
-def test_isocalendar_cftime(data):
+def test_calendar_cftime(data) -> None:
+    expected = data.time.values[0].calendar
+    assert data.time.dt.calendar == expected
 
-    with raises_regex(
-        AttributeError, "'CFTimeIndex' object has no attribute 'isocalendar'"
+
+@requires_cftime
+def test_calendar_cftime_2D(data) -> None:
+    # 2D np datetime:
+    data = xr.DataArray(
+        np.random.randint(1, 1000000, size=(4, 5)).astype("<M8[h]"), dims=("x", "y")
+    )
+    assert data.dt.calendar == "proleptic_gregorian"
+
+
+@requires_dask
+def test_calendar_dask() -> None:
+    import dask.array as da
+
+    # 3D lazy dask - np
+    data = xr.DataArray(
+        da.random.random_integers(1, 1000000, size=(4, 5, 6)).astype("<M8[h]"),
+        dims=("x", "y", "z"),
+    )
+    with raise_if_dask_computes():
+        assert data.dt.calendar == "proleptic_gregorian"
+
+
+@requires_dask
+@requires_cftime
+def test_calendar_dask_cftime() -> None:
+    from cftime import num2date
+
+    # 3D lazy dask
+    data = xr.DataArray(
+        num2date(
+            np.random.randint(1, 1000000, size=(4, 5, 6)),
+            "hours since 1970-01-01T00:00",
+            calendar="noleap",
+        ),
+        dims=("x", "y", "z"),
+    ).chunk()
+    with raise_if_dask_computes(max_computes=2):
+        assert data.dt.calendar == "noleap"
+
+
+@requires_cftime
+def test_isocalendar_cftime(data) -> None:
+
+    with pytest.raises(
+        AttributeError, match=r"'CFTimeIndex' object has no attribute 'isocalendar'"
     ):
         data.time.dt.isocalendar()
 
 
+@requires_cftime
+def test_date_cftime(data) -> None:
+
+    with pytest.raises(
+        AttributeError,
+        match=r"'CFTimeIndex' object has no attribute `date`. Consider using the floor method instead, for instance: `.time.dt.floor\('D'\)`.",
+    ):
+        data.time.dt.date()
+
+
 @requires_cftime
 @pytest.mark.filterwarnings("ignore::RuntimeWarning")
-def test_cftime_strftime_access(data):
-    """ compare cftime formatting against datetime formatting """
+def test_cftime_strftime_access(data) -> None:
+    """compare cftime formatting against datetime formatting"""
     date_format = "%Y%m%d%H"
     result = data.time.dt.strftime(date_format)
     datetime_array = xr.DataArray(
@@ -451,11 +500,9 @@ def test_cftime_strftime_access(data):
 @pytest.mark.parametrize(
     "field", ["year", "month", "day", "hour", "dayofyear", "dayofweek"]
 )
-def test_dask_field_access_1d(data, field):
+def test_dask_field_access_1d(data, field) -> None:
     import dask.array as da
 
-    if field == "dayofyear" or field == "dayofweek":
-        pytest.importorskip("cftime", minversion="1.0.2.1")
     expected = xr.DataArray(
         getattr(xr.coding.cftimeindex.CFTimeIndex(data.time.values), field),
         name=field,
@@ -473,11 +520,9 @@ def test_dask_field_access_1d(data, field):
 @pytest.mark.parametrize(
     "field", ["year", "month", "day", "hour", "dayofyear", "dayofweek"]
 )
-def test_dask_field_access(times_3d, data, field):
+def test_dask_field_access(times_3d, data, field) -> None:
     import dask.array as da
 
-    if field == "dayofyear" or field == "dayofweek":
-        pytest.importorskip("cftime", minversion="1.0.2.1")
     expected = xr.DataArray(
         getattr(
             xr.coding.cftimeindex.CFTimeIndex(times_3d.values.ravel()), field
@@ -501,24 +546,26 @@ def cftime_date_type(calendar):
 
 
 @requires_cftime
-def test_seasons(cftime_date_type):
-    dates = np.array([cftime_date_type(2000, month, 15) for month in range(1, 13)])
-    dates = xr.DataArray(dates)
-    seasons = [
-        "DJF",
-        "DJF",
-        "MAM",
-        "MAM",
-        "MAM",
-        "JJA",
-        "JJA",
-        "JJA",
-        "SON",
-        "SON",
-        "SON",
-        "DJF",
-    ]
-    seasons = xr.DataArray(seasons)
+def test_seasons(cftime_date_type) -> None:
+    dates = xr.DataArray(
+        np.array([cftime_date_type(2000, month, 15) for month in range(1, 13)])
+    )
+    seasons = xr.DataArray(
+        [
+            "DJF",
+            "DJF",
+            "MAM",
+            "MAM",
+            "MAM",
+            "JJA",
+            "JJA",
+            "JJA",
+            "SON",
+            "SON",
+            "SON",
+            "DJF",
+        ]
+    )
 
     assert_array_equal(seasons.values, dates.dt.season.values)
 
@@ -536,7 +583,9 @@ def cftime_rounding_dataarray(cftime_date_type):
 @requires_cftime
 @requires_dask
 @pytest.mark.parametrize("use_dask", [False, True])
-def test_cftime_floor_accessor(cftime_rounding_dataarray, cftime_date_type, use_dask):
+def test_cftime_floor_accessor(
+    cftime_rounding_dataarray, cftime_date_type, use_dask
+) -> None:
     import dask.array as da
 
     freq = "D"
@@ -567,7 +616,9 @@ def test_cftime_floor_accessor(cftime_rounding_dataarray, cftime_date_type, use_
 @requires_cftime
 @requires_dask
 @pytest.mark.parametrize("use_dask", [False, True])
-def test_cftime_ceil_accessor(cftime_rounding_dataarray, cftime_date_type, use_dask):
+def test_cftime_ceil_accessor(
+    cftime_rounding_dataarray, cftime_date_type, use_dask
+) -> None:
     import dask.array as da
 
     freq = "D"
@@ -598,7 +649,9 @@ def test_cftime_ceil_accessor(cftime_rounding_dataarray, cftime_date_type, use_d
 @requires_cftime
 @requires_dask
 @pytest.mark.parametrize("use_dask", [False, True])
-def test_cftime_round_accessor(cftime_rounding_dataarray, cftime_date_type, use_dask):
+def test_cftime_round_accessor(
+    cftime_rounding_dataarray, cftime_date_type, use_dask
+) -> None:
     import dask.array as da
 
     freq = "D"
