@@ -1,4 +1,5 @@
 from copy import deepcopy
+from typing import List
 
 import numpy as np
 import pandas as pd
@@ -12,13 +13,12 @@ from . import (
     assert_array_equal,
     assert_equal,
     assert_identical,
-    raises_regex,
     requires_dask,
 )
 from .test_dataset import create_test_data
 
 
-def test_concat_compat():
+def test_concat_compat() -> None:
     ds1 = Dataset(
         {
             "has_x_y": (("y", "x"), [[1, 2]]),
@@ -41,18 +41,20 @@ def test_concat_compat():
 
     for var in ["has_x", "no_x_y"]:
         assert "y" not in result[var].dims and "y" not in result[var].coords
-    with raises_regex(ValueError, "coordinates in some datasets but not others"):
+    with pytest.raises(
+        ValueError, match=r"coordinates in some datasets but not others"
+    ):
         concat([ds1, ds2], dim="q")
-    with raises_regex(ValueError, "'q' is not present in all datasets"):
+    with pytest.raises(ValueError, match=r"'q' is not present in all datasets"):
         concat([ds2, ds1], dim="q")
 
 
 class TestConcatDataset:
     @pytest.fixture
-    def data(self):
+    def data(self) -> Dataset:
         return create_test_data().drop_dims("dim3")
 
-    def rectify_dim_order(self, data, dataset):
+    def rectify_dim_order(self, data, dataset) -> Dataset:
         # return a new dataset with all variable dimensions transposed into
         # the order in which they are found in `data`
         return Dataset(
@@ -63,11 +65,11 @@ class TestConcatDataset:
 
     @pytest.mark.parametrize("coords", ["different", "minimal"])
     @pytest.mark.parametrize("dim", ["dim1", "dim2"])
-    def test_concat_simple(self, data, dim, coords):
+    def test_concat_simple(self, data, dim, coords) -> None:
         datasets = [g for _, g in data.groupby(dim, squeeze=False)]
         assert_identical(data, concat(datasets, dim, coords=coords))
 
-    def test_concat_merge_variables_present_in_some_datasets(self, data):
+    def test_concat_merge_variables_present_in_some_datasets(self, data) -> None:
         # coordinates present in some datasets but not others
         ds1 = Dataset(data_vars={"a": ("y", [0.1])}, coords={"x": 0.1})
         ds2 = Dataset(data_vars={"a": ("y", [0.2])}, coords={"z": 0.2})
@@ -83,7 +85,7 @@ class TestConcatDataset:
         expected = data.copy().assign(foo=data1.foo)
         assert_identical(expected, actual)
 
-    def test_concat_2(self, data):
+    def test_concat_2(self, data) -> None:
         dim = "dim2"
         datasets = [g for _, g in data.groupby(dim, squeeze=True)]
         concat_over = [k for k, v in data.coords.items() if dim in v.dims and k != dim]
@@ -92,7 +94,7 @@ class TestConcatDataset:
 
     @pytest.mark.parametrize("coords", ["different", "minimal", "all"])
     @pytest.mark.parametrize("dim", ["dim1", "dim2"])
-    def test_concat_coords_kwarg(self, data, dim, coords):
+    def test_concat_coords_kwarg(self, data, dim, coords) -> None:
         data = data.copy(deep=True)
         # make sure the coords argument behaves as expected
         data.coords["extra"] = ("dim4", np.arange(3))
@@ -106,7 +108,7 @@ class TestConcatDataset:
         else:
             assert_equal(data["extra"], actual["extra"])
 
-    def test_concat(self, data):
+    def test_concat(self, data) -> None:
         split_data = [
             data.isel(dim1=slice(3)),
             data.isel(dim1=3),
@@ -114,7 +116,7 @@ class TestConcatDataset:
         ]
         assert_identical(data, concat(split_data, "dim1"))
 
-    def test_concat_dim_precedence(self, data):
+    def test_concat_dim_precedence(self, data) -> None:
         # verify that the dim argument takes precedence over
         # concatenating dataset variables of the same name
         dim = (2 * data["dim1"]).rename("dim1")
@@ -123,14 +125,23 @@ class TestConcatDataset:
         expected["dim1"] = dim
         assert_identical(expected, concat(datasets, dim))
 
-    def test_concat_data_vars(self):
+    def test_concat_data_vars_typing(self) -> None:
+        # Testing typing, can be removed if the next function works with annotations.
         data = Dataset({"foo": ("x", np.random.randn(10))})
-        objs = [data.isel(x=slice(5)), data.isel(x=slice(5, None))]
+        objs: List[Dataset] = [data.isel(x=slice(5)), data.isel(x=slice(5, None))]
+        actual = concat(objs, dim="x", data_vars="minimal")
+        assert_identical(data, actual)
+
+    def test_concat_data_vars(self):
+        # TODO: annotating this func fails
+        data = Dataset({"foo": ("x", np.random.randn(10))})
+        objs: List[Dataset] = [data.isel(x=slice(5)), data.isel(x=slice(5, None))]
         for data_vars in ["minimal", "different", "all", [], ["foo"]]:
             actual = concat(objs, dim="x", data_vars=data_vars)
             assert_identical(data, actual)
 
     def test_concat_coords(self):
+        # TODO: annotating this func fails
         data = Dataset({"foo": ("x", np.random.randn(10))})
         expected = data.assign_coords(c=("x", [0] * 5 + [1] * 5))
         objs = [
@@ -141,10 +152,11 @@ class TestConcatDataset:
             actual = concat(objs, dim="x", coords=coords)
             assert_identical(expected, actual)
         for coords in ["minimal", []]:
-            with raises_regex(merge.MergeError, "conflicting values"):
+            with pytest.raises(merge.MergeError, match="conflicting values"):
                 concat(objs, dim="x", coords=coords)
 
     def test_concat_constant_index(self):
+        # TODO: annotating this func fails
         # GH425
         ds1 = Dataset({"foo": 1.5}, {"y": 1})
         ds2 = Dataset({"foo": 2.5}, {"y": 1})
@@ -152,12 +164,12 @@ class TestConcatDataset:
         for mode in ["different", "all", ["foo"]]:
             actual = concat([ds1, ds2], "y", data_vars=mode)
             assert_identical(expected, actual)
-        with raises_regex(merge.MergeError, "conflicting values"):
+        with pytest.raises(merge.MergeError, match="conflicting values"):
             # previously dim="y", and raised error which makes no sense.
             # "foo" has dimension "y" so minimal should concatenate it?
             concat([ds1, ds2], "new_dim", data_vars="minimal")
 
-    def test_concat_size0(self):
+    def test_concat_size0(self) -> None:
         data = create_test_data()
         split_data = [data.isel(dim1=slice(0, 0)), data]
         actual = concat(split_data, "dim1")
@@ -166,7 +178,7 @@ class TestConcatDataset:
         actual = concat(split_data[::-1], "dim1")
         assert_identical(data, actual)
 
-    def test_concat_autoalign(self):
+    def test_concat_autoalign(self) -> None:
         ds1 = Dataset({"foo": DataArray([1, 2], coords=[("x", [1, 2])])})
         ds2 = Dataset({"foo": DataArray([1, 2], coords=[("x", [1, 3])])})
         actual = concat([ds1, ds2], "y")
@@ -182,42 +194,47 @@ class TestConcatDataset:
         assert_identical(expected, actual)
 
     def test_concat_errors(self):
+        # TODO: annotating this func fails
         data = create_test_data()
         split_data = [data.isel(dim1=slice(3)), data.isel(dim1=slice(3, None))]
 
-        with raises_regex(ValueError, "must supply at least one"):
+        with pytest.raises(ValueError, match=r"must supply at least one"):
             concat([], "dim1")
 
-        with raises_regex(ValueError, "Cannot specify both .*='different'"):
+        with pytest.raises(ValueError, match=r"Cannot specify both .*='different'"):
             concat(
                 [data, data], dim="concat_dim", data_vars="different", compat="override"
             )
 
-        with raises_regex(ValueError, "must supply at least one"):
+        with pytest.raises(ValueError, match=r"must supply at least one"):
             concat([], "dim1")
 
-        with raises_regex(ValueError, "are not coordinates"):
+        with pytest.raises(ValueError, match=r"are not coordinates"):
             concat([data, data], "new_dim", coords=["not_found"])
 
-        with raises_regex(ValueError, "global attributes not"):
+        with pytest.raises(ValueError, match=r"global attributes not"):
             data0, data1 = deepcopy(split_data)
             data1.attrs["foo"] = "bar"
             concat([data0, data1], "dim1", compat="identical")
         assert_identical(data, concat([data0, data1], "dim1", compat="equals"))
 
-        with raises_regex(ValueError, "compat.* invalid"):
+        with pytest.raises(ValueError, match=r"compat.* invalid"):
             concat(split_data, "dim1", compat="foobar")
 
-        with raises_regex(ValueError, "unexpected value for"):
+        with pytest.raises(ValueError, match=r"unexpected value for"):
             concat([data, data], "new_dim", coords="foobar")
 
-        with raises_regex(ValueError, "coordinate in some datasets but not others"):
+        with pytest.raises(
+            ValueError, match=r"coordinate in some datasets but not others"
+        ):
             concat([Dataset({"x": 0}), Dataset({"x": [1]})], dim="z")
 
-        with raises_regex(ValueError, "coordinate in some datasets but not others"):
+        with pytest.raises(
+            ValueError, match=r"coordinate in some datasets but not others"
+        ):
             concat([Dataset({"x": 0}), Dataset({}, {"x": 1})], dim="z")
 
-    def test_concat_join_kwarg(self):
+    def test_concat_join_kwarg(self) -> None:
         ds1 = Dataset({"a": (("x", "y"), [[0]])}, coords={"x": [0], "y": [0]})
         ds2 = Dataset({"a": (("x", "y"), [[0]])}, coords={"x": [1], "y": [0.0001]})
 
@@ -242,7 +259,7 @@ class TestConcatDataset:
             coords={"x": [0, 1], "y": [0]},
         )
 
-        with raises_regex(ValueError, "indexes along dimension 'y'"):
+        with pytest.raises(ValueError, match=r"indexes along dimension 'y'"):
             actual = concat([ds1, ds2], join="exact", dim="x")
 
         for join in expected:
@@ -253,10 +270,10 @@ class TestConcatDataset:
         actual = concat(
             [ds1.drop_vars("x"), ds2.drop_vars("x")], join="override", dim="y"
         )
-        expected = Dataset(
+        expected2 = Dataset(
             {"a": (("x", "y"), np.array([0, 0], ndmin=2))}, coords={"y": [0, 0.0001]}
         )
-        assert_identical(actual, expected)
+        assert_identical(actual, expected2)
 
     @pytest.mark.parametrize(
         "combine_attrs, var1_attrs, var2_attrs, expected_attrs, expect_exception",
@@ -294,6 +311,13 @@ class TestConcatDataset:
                 {"a": 41, "c": 43, "d": 44},
                 False,
             ),
+            (
+                lambda attrs, context: {"a": -1, "b": 0, "c": 1} if any(attrs) else {},
+                {"a": 41, "b": 42, "c": 43},
+                {"b": 2, "c": 43, "d": 44},
+                {"a": -1, "b": 0, "c": 1},
+                False,
+            ),
         ],
     )
     def test_concat_combine_attrs_kwarg(
@@ -313,7 +337,6 @@ class TestConcatDataset:
 
             assert_identical(actual, expected)
 
-    @pytest.mark.skip(reason="not implemented, yet (see #4827)")
     @pytest.mark.parametrize(
         "combine_attrs, attrs1, attrs2, expected_attrs, expect_exception",
         [
@@ -350,6 +373,13 @@ class TestConcatDataset:
                 {"a": 41, "c": 43, "d": 44},
                 False,
             ),
+            (
+                lambda attrs, context: {"a": -1, "b": 0, "c": 1} if any(attrs) else {},
+                {"a": 41, "b": 42, "c": 43},
+                {"b": 2, "c": 43, "d": 44},
+                {"a": -1, "b": 0, "c": 1},
+                False,
+            ),
         ],
     )
     def test_concat_combine_attrs_kwarg_variables(
@@ -371,7 +401,7 @@ class TestConcatDataset:
 
             assert_identical(actual, expected)
 
-    def test_concat_promote_shape(self):
+    def test_concat_promote_shape(self) -> None:
         # mixed dims within variables
         objs = [Dataset({}, {"x": 0}), Dataset({"x": [1]})]
         actual = concat(objs, "x")
@@ -409,7 +439,7 @@ class TestConcatDataset:
         expected = Dataset({"z": (("x", "y"), [[-1], [1]])}, {"x": [0, 1], "y": [0]})
         assert_identical(actual, expected)
 
-    def test_concat_do_not_promote(self):
+    def test_concat_do_not_promote(self) -> None:
         # GH438
         objs = [
             Dataset({"y": ("t", [1])}, {"x": 1, "t": [0]}),
@@ -426,14 +456,14 @@ class TestConcatDataset:
         with pytest.raises(ValueError):
             concat(objs, "t", coords="minimal")
 
-    def test_concat_dim_is_variable(self):
+    def test_concat_dim_is_variable(self) -> None:
         objs = [Dataset({"x": 0}), Dataset({"x": 1})]
         coord = Variable("y", [3, 4])
         expected = Dataset({"x": ("y", [0, 1]), "y": [3, 4]})
         actual = concat(objs, coord)
         assert_identical(actual, expected)
 
-    def test_concat_multiindex(self):
+    def test_concat_multiindex(self) -> None:
         x = pd.MultiIndex.from_product([[1, 2, 3], ["a", "b"]])
         expected = Dataset({"x": x})
         actual = concat(
@@ -443,7 +473,7 @@ class TestConcatDataset:
         assert isinstance(actual.x.to_index(), pd.MultiIndex)
 
     @pytest.mark.parametrize("fill_value", [dtypes.NA, 2, 2.0, {"a": 2, "b": 1}])
-    def test_concat_fill_value(self, fill_value):
+    def test_concat_fill_value(self, fill_value) -> None:
         datasets = [
             Dataset({"a": ("x", [2, 3]), "b": ("x", [-2, 1]), "x": [1, 2]}),
             Dataset({"a": ("x", [1, 2]), "b": ("x", [3, -1]), "x": [0, 1]}),
@@ -469,7 +499,7 @@ class TestConcatDataset:
 
     @pytest.mark.parametrize("dtype", [str, bytes])
     @pytest.mark.parametrize("dim", ["x1", "x2"])
-    def test_concat_str_dtype(self, dtype, dim):
+    def test_concat_str_dtype(self, dtype, dim) -> None:
 
         data = np.arange(4).reshape([2, 2])
 
@@ -493,7 +523,7 @@ class TestConcatDataset:
 
 
 class TestConcatDataArray:
-    def test_concat(self):
+    def test_concat(self) -> None:
         ds = Dataset(
             {
                 "foo": (["x", "y"], np.random.random((2, 3))),
@@ -517,24 +547,24 @@ class TestConcatDataArray:
         stacked = concat(grouped, ds["x"])
         assert_identical(foo, stacked)
         # with an index as the 'dim' argument
-        stacked = concat(grouped, ds.indexes["x"])
+        stacked = concat(grouped, pd.Index(ds["x"], name="x"))
         assert_identical(foo, stacked)
 
-        actual = concat([foo[0], foo[1]], pd.Index([0, 1])).reset_coords(drop=True)
+        actual2 = concat([foo[0], foo[1]], pd.Index([0, 1])).reset_coords(drop=True)
         expected = foo[:2].rename({"x": "concat_dim"})
-        assert_identical(expected, actual)
+        assert_identical(expected, actual2)
 
-        actual = concat([foo[0], foo[1]], [0, 1]).reset_coords(drop=True)
+        actual3 = concat([foo[0], foo[1]], [0, 1]).reset_coords(drop=True)
         expected = foo[:2].rename({"x": "concat_dim"})
-        assert_identical(expected, actual)
+        assert_identical(expected, actual3)
 
-        with raises_regex(ValueError, "not identical"):
+        with pytest.raises(ValueError, match=r"not identical"):
             concat([foo, bar], dim="w", compat="identical")
 
-        with raises_regex(ValueError, "not a valid argument"):
+        with pytest.raises(ValueError, match=r"not a valid argument"):
             concat([foo, bar], dim="w", data_vars="minimal")
 
-    def test_concat_encoding(self):
+    def test_concat_encoding(self) -> None:
         # Regression test for GH1297
         ds = Dataset(
             {
@@ -550,7 +580,7 @@ class TestConcatDataArray:
         assert concat([ds, ds], dim="x").encoding == ds.encoding
 
     @requires_dask
-    def test_concat_lazy(self):
+    def test_concat_lazy(self) -> None:
         import dask.array as da
 
         arrays = [
@@ -565,7 +595,7 @@ class TestConcatDataArray:
         assert combined.dims == ("z", "x", "y")
 
     @pytest.mark.parametrize("fill_value", [dtypes.NA, 2, 2.0])
-    def test_concat_fill_value(self, fill_value):
+    def test_concat_fill_value(self, fill_value) -> None:
         foo = DataArray([1, 2], coords=[("x", [1, 2])])
         bar = DataArray([1, 2], coords=[("x", [1, 3])])
         if fill_value == dtypes.NA:
@@ -580,7 +610,7 @@ class TestConcatDataArray:
         actual = concat((foo, bar), dim="y", fill_value=fill_value)
         assert_identical(actual, expected)
 
-    def test_concat_join_kwarg(self):
+    def test_concat_join_kwarg(self) -> None:
         ds1 = Dataset(
             {"a": (("x", "y"), [[0]])}, coords={"x": [0], "y": [0]}
         ).to_array()
@@ -609,14 +639,14 @@ class TestConcatDataArray:
             coords={"x": [0, 1], "y": [0]},
         )
 
-        with raises_regex(ValueError, "indexes along dimension 'y'"):
+        with pytest.raises(ValueError, match=r"indexes along dimension 'y'"):
             actual = concat([ds1, ds2], join="exact", dim="x")
 
         for join in expected:
             actual = concat([ds1, ds2], join=join, dim="x")
             assert_equal(actual, expected[join].to_array())
 
-    def test_concat_combine_attrs_kwarg(self):
+    def test_concat_combine_attrs_kwarg(self) -> None:
         da1 = DataArray([0], coords=[("x", [0])], attrs={"b": 42})
         da2 = DataArray([0], coords=[("x", [1])], attrs={"b": 42, "c": 43})
 
@@ -629,9 +659,9 @@ class TestConcatDataArray:
             [0, 0], coords=[("x", [0, 1])], attrs={"b": 42}
         )
 
-        with raises_regex(ValueError, "combine_attrs='identical'"):
+        with pytest.raises(ValueError, match=r"combine_attrs='identical'"):
             actual = concat([da1, da2], dim="x", combine_attrs="identical")
-        with raises_regex(ValueError, "combine_attrs='no_conflicts'"):
+        with pytest.raises(ValueError, match=r"combine_attrs='no_conflicts'"):
             da3 = da2.copy(deep=True)
             da3.attrs["b"] = 44
             actual = concat([da1, da3], dim="x", combine_attrs="no_conflicts")
@@ -642,7 +672,7 @@ class TestConcatDataArray:
 
     @pytest.mark.parametrize("dtype", [str, bytes])
     @pytest.mark.parametrize("dim", ["x1", "x2"])
-    def test_concat_str_dtype(self, dtype, dim):
+    def test_concat_str_dtype(self, dtype, dim) -> None:
 
         data = np.arange(4).reshape([2, 2])
 
@@ -660,10 +690,19 @@ class TestConcatDataArray:
 
         assert np.issubdtype(actual.x2.dtype, dtype)
 
+    def test_concat_coord_name(self) -> None:
+
+        da = DataArray([0], dims="a")
+        da_concat = concat([da, da], dim=DataArray([0, 1], dims="b"))
+        assert list(da_concat.coords) == ["b"]
+
+        da_concat_std = concat([da, da], dim=DataArray([0, 1]))
+        assert list(da_concat_std.coords) == ["dim_0"]
+
 
 @pytest.mark.parametrize("attr1", ({"a": {"meta": [10, 20, 30]}}, {"a": [1, 2, 3]}, {}))
 @pytest.mark.parametrize("attr2", ({"a": [1, 2, 3]}, {}))
-def test_concat_attrs_first_variable(attr1, attr2):
+def test_concat_attrs_first_variable(attr1, attr2) -> None:
 
     arrs = [
         DataArray([[1], [2]], dims=["x", "y"], attrs=attr1),
@@ -675,6 +714,7 @@ def test_concat_attrs_first_variable(attr1, attr2):
 
 
 def test_concat_merge_single_non_dim_coord():
+    # TODO: annotating this func fails
     da1 = DataArray([1, 2, 3], dims="x", coords={"x": [1, 2, 3], "y": 1})
     da2 = DataArray([4, 5, 6], dims="x", coords={"x": [4, 5, 6]})
 
@@ -684,18 +724,18 @@ def test_concat_merge_single_non_dim_coord():
         actual = concat([da1, da2], "x", coords=coords)
         assert_identical(actual, expected)
 
-    with raises_regex(ValueError, "'y' is not present in all datasets."):
+    with pytest.raises(ValueError, match=r"'y' is not present in all datasets."):
         concat([da1, da2], dim="x", coords="all")
 
     da1 = DataArray([1, 2, 3], dims="x", coords={"x": [1, 2, 3], "y": 1})
     da2 = DataArray([4, 5, 6], dims="x", coords={"x": [4, 5, 6]})
     da3 = DataArray([7, 8, 9], dims="x", coords={"x": [7, 8, 9], "y": 1})
     for coords in ["different", "all"]:
-        with raises_regex(ValueError, "'y' not present in all datasets"):
+        with pytest.raises(ValueError, match=r"'y' not present in all datasets"):
             concat([da1, da2, da3], dim="x")
 
 
-def test_concat_preserve_coordinate_order():
+def test_concat_preserve_coordinate_order() -> None:
     x = np.arange(0, 5)
     y = np.arange(0, 10)
     time = np.arange(0, 4)
@@ -726,3 +766,20 @@ def test_concat_preserve_coordinate_order():
     for act, exp in zip(actual.coords, expected.coords):
         assert act == exp
         assert_identical(actual.coords[act], expected.coords[exp])
+
+
+def test_concat_typing_check() -> None:
+    ds = Dataset({"foo": 1}, {"bar": 2})
+    da = Dataset({"foo": 3}, {"bar": 4}).to_array(dim="foo")
+
+    # concatenate a list of non-homogeneous types must raise TypeError
+    with pytest.raises(
+        TypeError,
+        match="The elements in the input list need to be either all 'Dataset's or all 'DataArray's",
+    ):
+        concat([ds, da], dim="foo")
+    with pytest.raises(
+        TypeError,
+        match="The elements in the input list need to be either all 'Dataset's or all 'DataArray's",
+    ):
+        concat([da, ds], dim="foo")
