@@ -132,13 +132,13 @@ def test_dask_distributed_read_netcdf_integration_test(
 @requires_zarr
 @pytest.mark.parametrize("consolidated", [True, False])
 @pytest.mark.parametrize("compute", [True, False])
-def test_dask_distributed_zarr_integration_test(loop, consolidated, compute):
+def test_dask_distributed_zarr_integration_test(loop, consolidated, compute) -> None:
     if consolidated:
         pytest.importorskip("zarr", minversion="2.2.1.dev2")
         write_kwargs = {"consolidated": True}
         read_kwargs = {"backend_kwargs": {"consolidated": True}}
     else:
-        write_kwargs = read_kwargs = {}
+        write_kwargs = read_kwargs = {}  # type: ignore
     chunks = {"dim1": 4, "dim2": 3, "dim3": 5}
     with cluster() as (s, [a, b]):
         with Client(s["address"], loop=loop):
@@ -160,10 +160,10 @@ def test_dask_distributed_zarr_integration_test(loop, consolidated, compute):
 
 
 @requires_rasterio
-def test_dask_distributed_rasterio_integration_test(loop):
+def test_dask_distributed_rasterio_integration_test(loop) -> None:
     with create_tmp_geotiff() as (tmp_file, expected):
         with cluster() as (s, [a, b]):
-            with Client(s["address"], loop=loop):
+            with pytest.warns(DeprecationWarning), Client(s["address"], loop=loop):
                 da_tiff = xr.open_rasterio(tmp_file, chunks={"band": 1})
                 assert isinstance(da_tiff.data, da.Array)
                 actual = da_tiff.compute()
@@ -171,7 +171,8 @@ def test_dask_distributed_rasterio_integration_test(loop):
 
 
 @requires_cfgrib
-def test_dask_distributed_cfgrib_integration_test(loop):
+@pytest.mark.filterwarnings("ignore:deallocating CachingFileManager")
+def test_dask_distributed_cfgrib_integration_test(loop) -> None:
     with cluster() as (s, [a, b]):
         with Client(s["address"], loop=loop):
             with open_example_dataset(
@@ -183,12 +184,9 @@ def test_dask_distributed_cfgrib_integration_test(loop):
                     assert_allclose(actual, expected)
 
 
-@pytest.mark.skipif(
-    distributed.__version__ <= "1.19.3",
-    reason="Need recent distributed version to clean up get",
-)
-@gen_cluster(client=True, timeout=None)
-async def test_async(c, s, a, b):
+@pytest.mark.xfail(reason="https://github.com/pydata/xarray/pull/6211")
+@gen_cluster(client=True)
+async def test_async(c, s, a, b) -> None:
     x = create_test_data()
     assert not dask.is_dask_collection(x)
     y = x.chunk({"dim2": 4}) + 10
@@ -215,12 +213,13 @@ async def test_async(c, s, a, b):
     assert s.tasks
 
 
-def test_hdf5_lock():
+def test_hdf5_lock() -> None:
     assert isinstance(HDF5_LOCK, dask.utils.SerializableLock)
 
 
+@pytest.mark.xfail(reason="https://github.com/pydata/xarray/pull/6211")
 @gen_cluster(client=True)
-async def test_serializable_locks(c, s, a, b):
+async def test_serializable_locks(c, s, a, b) -> None:
     def f(x, lock=None):
         with lock:
             return x + 1
