@@ -7,7 +7,7 @@ import pytest
 from xarray import DataArray, Dataset, Variable
 from xarray.core import indexing, nputils
 
-from . import IndexerMaker, ReturnItem, assert_array_equal, raises_regex
+from . import IndexerMaker, ReturnItem, assert_array_equal
 
 B = IndexerMaker(indexing.BasicIndexer)
 
@@ -18,7 +18,7 @@ class TestIndexers:
         x[i] = 0
         return x
 
-    def test_expanded_indexer(self):
+    def test_expanded_indexer(self) -> None:
         x = np.random.randn(10, 11, 12, 13, 14)
         y = np.arange(5)
         arr = ReturnItem()
@@ -37,21 +37,10 @@ class TestIndexers:
             j = indexing.expanded_indexer(i, x.ndim)
             assert_array_equal(x[i], x[j])
             assert_array_equal(self.set_to_zero(x, i), self.set_to_zero(x, j))
-        with raises_regex(IndexError, "too many indices"):
+        with pytest.raises(IndexError, match=r"too many indices"):
             indexing.expanded_indexer(arr[1, 2, 3], 2)
 
-    def test_asarray_tuplesafe(self):
-        res = indexing._asarray_tuplesafe(("a", 1))
-        assert isinstance(res, np.ndarray)
-        assert res.ndim == 0
-        assert res.item() == ("a", 1)
-
-        res = indexing._asarray_tuplesafe([(0,), (1,)])
-        assert res.shape == (2,)
-        assert res[0] == (0,)
-        assert res[1] == (1,)
-
-    def test_stacked_multiindex_min_max(self):
+    def test_stacked_multiindex_min_max(self) -> None:
         data = np.random.randn(3, 23, 4)
         da = DataArray(
             data,
@@ -66,64 +55,38 @@ class TestIndexers:
         assert_array_equal(da2.loc["a", s.max()], data[2, 22, 0])
         assert_array_equal(da2.loc["b", s.min()], data[0, 0, 1])
 
-    def test_convert_label_indexer(self):
-        # TODO: add tests that aren't just for edge cases
-        index = pd.Index([1, 2, 3])
-        with raises_regex(KeyError, "not all values found"):
-            indexing.convert_label_indexer(index, [0])
-        with pytest.raises(KeyError):
-            indexing.convert_label_indexer(index, 0)
-        with raises_regex(ValueError, "does not have a MultiIndex"):
-            indexing.convert_label_indexer(index, {"one": 0})
-
+    def test_group_indexers_by_index(self) -> None:
         mindex = pd.MultiIndex.from_product([["a", "b"], [1, 2]], names=("one", "two"))
-        with raises_regex(KeyError, "not all values found"):
-            indexing.convert_label_indexer(mindex, [0])
-        with pytest.raises(KeyError):
-            indexing.convert_label_indexer(mindex, 0)
-        with pytest.raises(ValueError):
-            indexing.convert_label_indexer(index, {"three": 0})
-        with pytest.raises(IndexError):
-            indexing.convert_label_indexer(mindex, (slice(None), 1, "no_level"))
+        data = DataArray(
+            np.zeros((4, 2, 2)), coords={"x": mindex, "y": [1, 2]}, dims=("x", "y", "z")
+        )
+        data.coords["y2"] = ("y", [2.0, 3.0])
 
-    def test_convert_label_indexer_datetime(self):
-        index = pd.to_datetime(["2000-01-01", "2001-01-01", "2002-01-01"])
-        actual = indexing.convert_label_indexer(index, "2001-01-01")
-        expected = (1, None)
-        assert actual == expected
+        indexes, grouped_indexers = indexing.group_indexers_by_index(
+            data, {"z": 0, "one": "a", "two": 1, "y": 0}
+        )
+        assert indexes == {"x": data.xindexes["x"], "y": data.xindexes["y"]}
+        assert grouped_indexers == {
+            "x": {"one": "a", "two": 1},
+            "y": {"y": 0},
+            None: {"z": 0},
+        }
 
-        actual = indexing.convert_label_indexer(index, index.to_numpy()[1])
-        assert actual == expected
+        with pytest.raises(KeyError, match=r"no index found for coordinate y2"):
+            indexing.group_indexers_by_index(data, {"y2": 2.0})
+        with pytest.raises(KeyError, match=r"w is not a valid dimension or coordinate"):
+            indexing.group_indexers_by_index(data, {"w": "a"})
+        with pytest.raises(ValueError, match=r"cannot supply.*"):
+            indexing.group_indexers_by_index(data, {"z": 1}, method="nearest")
 
-    def test_convert_unsorted_datetime_index_raises(self):
-        index = pd.to_datetime(["2001", "2000", "2002"])
-        with pytest.raises(KeyError):
-            # pandas will try to convert this into an array indexer. We should
-            # raise instead, so we can be sure the result of indexing with a
-            # slice is always a view.
-            indexing.convert_label_indexer(index, slice("2001", "2002"))
-
-    def test_get_dim_indexers(self):
-        mindex = pd.MultiIndex.from_product([["a", "b"], [1, 2]], names=("one", "two"))
-        mdata = DataArray(range(4), [("x", mindex)])
-
-        dim_indexers = indexing.get_dim_indexers(mdata, {"one": "a", "two": 1})
-        assert dim_indexers == {"x": {"one": "a", "two": 1}}
-
-        with raises_regex(ValueError, "cannot combine"):
-            indexing.get_dim_indexers(mdata, {"x": "a", "two": 1})
-
-        with raises_regex(ValueError, "do not exist"):
-            indexing.get_dim_indexers(mdata, {"y": "a"})
-
-        with raises_regex(ValueError, "do not exist"):
-            indexing.get_dim_indexers(mdata, {"four": 1})
-
-    def test_remap_label_indexers(self):
-        def test_indexer(data, x, expected_pos, expected_idx=None):
-            pos, idx = indexing.remap_label_indexers(data, {"x": x})
+    def test_remap_label_indexers(self) -> None:
+        def test_indexer(data, x, expected_pos, expected_idx=None) -> None:
+            pos, new_idx_vars = indexing.remap_label_indexers(data, {"x": x})
+            idx, _ = new_idx_vars.get("x", (None, None))
+            if idx is not None:
+                idx = idx.to_pandas_index()
             assert_array_equal(pos.get("x"), expected_pos)
-            assert_array_equal(idx.get("x"), expected_idx)
+            assert_array_equal(idx, expected_idx)
 
         data = Dataset({"x": ("x", [1, 2, 3])})
         mindex = pd.MultiIndex.from_product(
@@ -176,7 +139,7 @@ class TestIndexers:
             pd.MultiIndex.from_product([[1, 2], [-1, -2]]),
         )
 
-    def test_read_only_view(self):
+    def test_read_only_view(self) -> None:
 
         arr = DataArray(
             np.random.rand(3, 3),
@@ -190,7 +153,7 @@ class TestIndexers:
 
 
 class TestLazyArray:
-    def test_slice_slice(self):
+    def test_slice_slice(self) -> None:
         arr = ReturnItem()
         for size in [100, 99]:
             # We test even/odd size cases
@@ -220,11 +183,11 @@ class TestLazyArray:
                     actual = x[new_slice]
                     assert_array_equal(expected, actual)
 
-    def test_lazily_indexed_array(self):
+    def test_lazily_indexed_array(self) -> None:
         original = np.random.rand(10, 20, 30)
         x = indexing.NumpyIndexingAdapter(original)
         v = Variable(["i", "j", "k"], original)
-        lazy = indexing.LazilyOuterIndexedArray(x)
+        lazy = indexing.LazilyIndexedArray(x)
         v_lazy = Variable(["i", "j", "k"], lazy)
         arr = ReturnItem()
         # test orthogonally applied indexers
@@ -244,9 +207,7 @@ class TestLazyArray:
                     ]:
                         assert expected.shape == actual.shape
                         assert_array_equal(expected, actual)
-                        assert isinstance(
-                            actual._data, indexing.LazilyOuterIndexedArray
-                        )
+                        assert isinstance(actual._data, indexing.LazilyIndexedArray)
 
                         # make sure actual.key is appropriate type
                         if all(
@@ -267,33 +228,33 @@ class TestLazyArray:
             ([0, 3, 5], arr[:2]),
         ]
         for i, j in indexers:
-            expected = v[i][j]
+            expected_b = v[i][j]
             actual = v_lazy[i][j]
-            assert expected.shape == actual.shape
-            assert_array_equal(expected, actual)
+            assert expected_b.shape == actual.shape
+            assert_array_equal(expected_b, actual)
 
             # test transpose
             if actual.ndim > 1:
                 order = np.random.choice(actual.ndim, actual.ndim)
                 order = np.array(actual.dims)
                 transposed = actual.transpose(*order)
-                assert_array_equal(expected.transpose(*order), transposed)
+                assert_array_equal(expected_b.transpose(*order), transposed)
                 assert isinstance(
                     actual._data,
                     (
                         indexing.LazilyVectorizedIndexedArray,
-                        indexing.LazilyOuterIndexedArray,
+                        indexing.LazilyIndexedArray,
                     ),
                 )
 
-            assert isinstance(actual._data, indexing.LazilyOuterIndexedArray)
+            assert isinstance(actual._data, indexing.LazilyIndexedArray)
             assert isinstance(actual._data.array, indexing.NumpyIndexingAdapter)
 
-    def test_vectorized_lazily_indexed_array(self):
+    def test_vectorized_lazily_indexed_array(self) -> None:
         original = np.random.rand(10, 20, 30)
         x = indexing.NumpyIndexingAdapter(original)
         v_eager = Variable(["i", "j", "k"], x)
-        lazy = indexing.LazilyOuterIndexedArray(x)
+        lazy = indexing.LazilyIndexedArray(x)
         v_lazy = Variable(["i", "j", "k"], lazy)
         arr = ReturnItem()
 
@@ -306,7 +267,7 @@ class TestLazyArray:
                     actual._data,
                     (
                         indexing.LazilyVectorizedIndexedArray,
-                        indexing.LazilyOuterIndexedArray,
+                        indexing.LazilyIndexedArray,
                     ),
                 )
                 assert_array_equal(expected, actual)
@@ -339,14 +300,14 @@ class TestLazyArray:
 
 
 class TestCopyOnWriteArray:
-    def test_setitem(self):
+    def test_setitem(self) -> None:
         original = np.arange(10)
         wrapped = indexing.CopyOnWriteArray(original)
         wrapped[B[:]] = 0
         assert_array_equal(original, np.arange(10))
         assert_array_equal(wrapped, np.zeros(10))
 
-    def test_sub_array(self):
+    def test_sub_array(self) -> None:
         original = np.arange(10)
         wrapped = indexing.CopyOnWriteArray(original)
         child = wrapped[B[:5]]
@@ -356,41 +317,41 @@ class TestCopyOnWriteArray:
         assert_array_equal(wrapped, np.arange(10))
         assert_array_equal(child, np.zeros(5))
 
-    def test_index_scalar(self):
+    def test_index_scalar(self) -> None:
         # regression test for GH1374
         x = indexing.CopyOnWriteArray(np.array(["foo", "bar"]))
         assert np.array(x[B[0]][B[()]]) == "foo"
 
 
 class TestMemoryCachedArray:
-    def test_wrapper(self):
-        original = indexing.LazilyOuterIndexedArray(np.arange(10))
+    def test_wrapper(self) -> None:
+        original = indexing.LazilyIndexedArray(np.arange(10))
         wrapped = indexing.MemoryCachedArray(original)
         assert_array_equal(wrapped, np.arange(10))
         assert isinstance(wrapped.array, indexing.NumpyIndexingAdapter)
 
-    def test_sub_array(self):
-        original = indexing.LazilyOuterIndexedArray(np.arange(10))
+    def test_sub_array(self) -> None:
+        original = indexing.LazilyIndexedArray(np.arange(10))
         wrapped = indexing.MemoryCachedArray(original)
         child = wrapped[B[:5]]
         assert isinstance(child, indexing.MemoryCachedArray)
         assert_array_equal(child, np.arange(5))
         assert isinstance(child.array, indexing.NumpyIndexingAdapter)
-        assert isinstance(wrapped.array, indexing.LazilyOuterIndexedArray)
+        assert isinstance(wrapped.array, indexing.LazilyIndexedArray)
 
-    def test_setitem(self):
+    def test_setitem(self) -> None:
         original = np.arange(10)
         wrapped = indexing.MemoryCachedArray(original)
         wrapped[B[:]] = 0
         assert_array_equal(original, np.zeros(10))
 
-    def test_index_scalar(self):
+    def test_index_scalar(self) -> None:
         # regression test for GH1374
         x = indexing.MemoryCachedArray(np.array(["foo", "bar"]))
         assert np.array(x[B[0]][B[()]]) == "foo"
 
 
-def test_base_explicit_indexer():
+def test_base_explicit_indexer() -> None:
     with pytest.raises(TypeError):
         indexing.ExplicitIndexer(())
 
@@ -406,7 +367,7 @@ def test_base_explicit_indexer():
     "indexer_cls",
     [indexing.BasicIndexer, indexing.OuterIndexer, indexing.VectorizedIndexer],
 )
-def test_invalid_for_all(indexer_cls):
+def test_invalid_for_all(indexer_cls) -> None:
     with pytest.raises(TypeError):
         indexer_cls(None)
     with pytest.raises(TypeError):
@@ -448,7 +409,7 @@ def check_array2d(indexer_cls):
     np.testing.assert_array_equal(value, array)
 
 
-def test_basic_indexer():
+def test_basic_indexer() -> None:
     check_integer(indexing.BasicIndexer)
     check_slice(indexing.BasicIndexer)
     with pytest.raises(TypeError):
@@ -457,7 +418,7 @@ def test_basic_indexer():
         check_array2d(indexing.BasicIndexer)
 
 
-def test_outer_indexer():
+def test_outer_indexer() -> None:
     check_integer(indexing.OuterIndexer)
     check_slice(indexing.OuterIndexer)
     check_array1d(indexing.OuterIndexer)
@@ -465,13 +426,13 @@ def test_outer_indexer():
         check_array2d(indexing.OuterIndexer)
 
 
-def test_vectorized_indexer():
+def test_vectorized_indexer() -> None:
     with pytest.raises(TypeError):
         check_integer(indexing.VectorizedIndexer)
     check_slice(indexing.VectorizedIndexer)
     check_array1d(indexing.VectorizedIndexer)
     check_array2d(indexing.VectorizedIndexer)
-    with raises_regex(ValueError, "numbers of dimensions"):
+    with pytest.raises(ValueError, match=r"numbers of dimensions"):
         indexing.VectorizedIndexer(
             (np.array(1, dtype=np.int64), np.arange(5, dtype=np.int64))
         )
@@ -489,7 +450,7 @@ class Test_vectorized_indexer:
             slice(None),
         ]
 
-    def test_arrayize_vectorized_indexer(self):
+    def test_arrayize_vectorized_indexer(self) -> None:
         for i, j, k in itertools.product(self.indexers, repeat=3):
             vindex = indexing.VectorizedIndexer((i, j, k))
             vindex_array = indexing._arrayize_vectorized_indexer(
@@ -569,7 +530,7 @@ def get_indexers(shape, mode):
 @pytest.mark.parametrize(
     "sl", [slice(1, -1, 1), slice(None, -1, 2), slice(-1, 1, -1), slice(-1, 1, -2)]
 )
-def test_decompose_slice(size, sl):
+def test_decompose_slice(size, sl) -> None:
     x = np.arange(size)
     slice1, slice2 = indexing._decompose_slice(sl, size)
     expected = x[sl]
@@ -601,7 +562,7 @@ def test_decompose_slice(size, sl):
         indexing.IndexingSupport.VECTORIZED,
     ],
 )
-def test_decompose_indexers(shape, indexer_mode, indexing_support):
+def test_decompose_indexers(shape, indexer_mode, indexing_support) -> None:
     data = np.random.randn(*shape)
     indexer = get_indexers(shape, indexer_mode)
 
@@ -619,7 +580,7 @@ def test_decompose_indexers(shape, indexer_mode, indexing_support):
         np.testing.assert_array_equal(expected, array)
 
 
-def test_implicit_indexing_adapter():
+def test_implicit_indexing_adapter() -> None:
     array = np.arange(10, dtype=np.int64)
     implicit = indexing.ImplicitToExplicitIndexingAdapter(
         indexing.NumpyIndexingAdapter(array), indexing.BasicIndexer
@@ -628,7 +589,7 @@ def test_implicit_indexing_adapter():
     np.testing.assert_array_equal(array, implicit[:])
 
 
-def test_implicit_indexing_adapter_copy_on_write():
+def test_implicit_indexing_adapter_copy_on_write() -> None:
     array = np.arange(10, dtype=np.int64)
     implicit = indexing.ImplicitToExplicitIndexingAdapter(
         indexing.CopyOnWriteArray(array)
@@ -636,7 +597,7 @@ def test_implicit_indexing_adapter_copy_on_write():
     assert isinstance(implicit[:], indexing.ImplicitToExplicitIndexingAdapter)
 
 
-def test_outer_indexer_consistency_with_broadcast_indexes_vectorized():
+def test_outer_indexer_consistency_with_broadcast_indexes_vectorized() -> None:
     def nonzero(x):
         if isinstance(x, np.ndarray) and x.dtype.kind == "b":
             x = x.nonzero()[0]
@@ -674,7 +635,7 @@ def test_outer_indexer_consistency_with_broadcast_indexes_vectorized():
         np.testing.assert_array_equal(actual_data, expected_data)
 
 
-def test_create_mask_outer_indexer():
+def test_create_mask_outer_indexer() -> None:
     indexer = indexing.OuterIndexer((np.array([0, -1, 2]),))
     expected = np.array([False, True, False])
     actual = indexing.create_mask(indexer, (5,))
@@ -686,7 +647,7 @@ def test_create_mask_outer_indexer():
     np.testing.assert_array_equal(expected, actual)
 
 
-def test_create_mask_vectorized_indexer():
+def test_create_mask_vectorized_indexer() -> None:
     indexer = indexing.VectorizedIndexer((np.array([0, -1, 2]), np.array([0, 1, -1])))
     expected = np.array([False, True, True])
     actual = indexing.create_mask(indexer, (5,))
@@ -700,7 +661,7 @@ def test_create_mask_vectorized_indexer():
     np.testing.assert_array_equal(expected, actual)
 
 
-def test_create_mask_basic_indexer():
+def test_create_mask_basic_indexer() -> None:
     indexer = indexing.BasicIndexer((-1,))
     actual = indexing.create_mask(indexer, (3,))
     np.testing.assert_array_equal(True, actual)
@@ -710,7 +671,7 @@ def test_create_mask_basic_indexer():
     np.testing.assert_array_equal(False, actual)
 
 
-def test_create_mask_dask():
+def test_create_mask_dask() -> None:
     da = pytest.importorskip("dask.array")
 
     indexer = indexing.OuterIndexer((1, slice(2), np.array([0, -1, 2])))
@@ -721,22 +682,22 @@ def test_create_mask_dask():
     assert actual.chunks == ((1, 1), (2, 1))
     np.testing.assert_array_equal(expected, actual)
 
-    indexer = indexing.VectorizedIndexer(
+    indexer_vec = indexing.VectorizedIndexer(
         (np.array([0, -1, 2]), slice(None), np.array([0, 1, -1]))
     )
     expected = np.array([[False, True, True]] * 2).T
     actual = indexing.create_mask(
-        indexer, (5, 2), da.empty((3, 2), chunks=((3,), (2,)))
+        indexer_vec, (5, 2), da.empty((3, 2), chunks=((3,), (2,)))
     )
     assert isinstance(actual, da.Array)
     np.testing.assert_array_equal(expected, actual)
 
     with pytest.raises(ValueError):
-        indexing.create_mask(indexer, (5, 2), da.empty((5,), chunks=(1,)))
+        indexing.create_mask(indexer_vec, (5, 2), da.empty((5,), chunks=(1,)))
 
 
-def test_create_mask_error():
-    with raises_regex(TypeError, "unexpected key type"):
+def test_create_mask_error() -> None:
+    with pytest.raises(TypeError, match=r"unexpected key type"):
         indexing.create_mask((1, 2), (3, 4))
 
 
@@ -752,6 +713,19 @@ def test_create_mask_error():
         (np.array([0, -1, -1, -1, 1]), np.array([0, 0, 0, 0, 1])),
     ],
 )
-def test_posify_mask_subindexer(indices, expected):
+def test_posify_mask_subindexer(indices, expected) -> None:
     actual = indexing._posify_mask_subindexer(indices)
     np.testing.assert_array_equal(expected, actual)
+
+
+def test_indexing_1d_object_array() -> None:
+    items = (np.arange(3), np.arange(6))
+    arr = DataArray(np.array(items, dtype=object))
+
+    actual = arr[0]
+
+    expected_data = np.empty((), dtype=object)
+    expected_data[()] = items[0]
+    expected = DataArray(expected_data)
+
+    assert [actual.data.item()] == [expected.data.item()]
