@@ -1,10 +1,12 @@
+from importlib.metadata import EntryPoint
 from unittest import mock
 
-import pkg_resources
 import pytest
 
 from xarray.backends import common, plugins
 
+importlib_metadata_mock = "importlib.metadata"
+
 
 class DummyBackendEntrypointArgs(common.BackendEntrypoint):
     def open_dataset(filename_or_obj, *args):
@@ -29,23 +31,36 @@ class DummyBackendEntrypoint2(common.BackendEntrypoint):
 @pytest.fixture
 def dummy_duplicated_entrypoints():
     specs = [
-        "engine1 = xarray.tests.test_plugins:backend_1",
-        "engine1 = xarray.tests.test_plugins:backend_2",
-        "engine2 = xarray.tests.test_plugins:backend_1",
-        "engine2 = xarray.tests.test_plugins:backend_2",
+        ["engine1", "xarray.tests.test_plugins:backend_1", "xarray.backends"],
+        ["engine1", "xarray.tests.test_plugins:backend_2", "xarray.backends"],
+        ["engine2", "xarray.tests.test_plugins:backend_1", "xarray.backends"],
+        ["engine2", "xarray.tests.test_plugins:backend_2", "xarray.backends"],
     ]
-    eps = [pkg_resources.EntryPoint.parse(spec) for spec in specs]
+    eps = [EntryPoint(name, value, group) for name, value, group in specs]
     return eps
 
 
 @pytest.mark.filterwarnings("ignore:Found")
-def test_remove_duplicates(dummy_duplicated_entrypoints):
+def test_remove_duplicates(dummy_duplicated_entrypoints) -> None:
     with pytest.warns(RuntimeWarning):
         entrypoints = plugins.remove_duplicates(dummy_duplicated_entrypoints)
     assert len(entrypoints) == 2
 
 
-def test_remove_duplicates_warnings(dummy_duplicated_entrypoints):
+def test_broken_plugin() -> None:
+    broken_backend = EntryPoint(
+        "broken_backend",
+        "xarray.tests.test_plugins:backend_1",
+        "xarray.backends",
+    )
+    with pytest.warns(RuntimeWarning) as record:
+        _ = plugins.build_engines([broken_backend])
+    assert len(record) == 1
+    message = str(record[0].message)
+    assert "Engine 'broken_backend'" in message
+
+
+def test_remove_duplicates_warnings(dummy_duplicated_entrypoints) -> None:
 
     with pytest.warns(RuntimeWarning) as record:
         _ = plugins.remove_duplicates(dummy_duplicated_entrypoints)
@@ -57,19 +72,21 @@ def test_remove_duplicates_warnings(dummy_duplicated_entrypoints):
     assert "entrypoints" in message1
 
 
-@mock.patch("pkg_resources.EntryPoint.load", mock.MagicMock(return_value=None))
-def test_backends_dict_from_pkg():
+@mock.patch(
+    f"{importlib_metadata_mock}.EntryPoint.load", mock.MagicMock(return_value=None)
+)
+def test_backends_dict_from_pkg() -> None:
     specs = [
-        "engine1 = xarray.tests.test_plugins:backend_1",
-        "engine2 = xarray.tests.test_plugins:backend_2",
+        ["engine1", "xarray.tests.test_plugins:backend_1", "xarray.backends"],
+        ["engine2", "xarray.tests.test_plugins:backend_2", "xarray.backends"],
     ]
-    entrypoints = [pkg_resources.EntryPoint.parse(spec) for spec in specs]
+    entrypoints = [EntryPoint(name, value, group) for name, value, group in specs]
     engines = plugins.backends_dict_from_pkg(entrypoints)
     assert len(engines) == 2
-    assert engines.keys() == set(("engine1", "engine2"))
+    assert engines.keys() == {"engine1", "engine2"}
 
 
-def test_set_missing_parameters():
+def test_set_missing_parameters() -> None:
     backend_1 = DummyBackendEntrypoint1
     backend_2 = DummyBackendEntrypoint2
     backend_2.open_dataset_parameters = ("filename_or_obj",)
@@ -85,30 +102,30 @@ def test_set_missing_parameters():
     plugins.set_missing_parameters({"engine": backend})
     assert backend.open_dataset_parameters == ("filename_or_obj", "decoder")
 
-    backend = DummyBackendEntrypointArgs()
-    backend.open_dataset_parameters = ("filename_or_obj", "decoder")
-    plugins.set_missing_parameters({"engine": backend})
-    assert backend.open_dataset_parameters == ("filename_or_obj", "decoder")
+    backend_args = DummyBackendEntrypointArgs()
+    backend_args.open_dataset_parameters = ("filename_or_obj", "decoder")
+    plugins.set_missing_parameters({"engine": backend_args})
+    assert backend_args.open_dataset_parameters == ("filename_or_obj", "decoder")
 
 
-def test_set_missing_parameters_raise_error():
+def test_set_missing_parameters_raise_error() -> None:
 
     backend = DummyBackendEntrypointKwargs()
     with pytest.raises(TypeError):
         plugins.set_missing_parameters({"engine": backend})
 
-    backend = DummyBackendEntrypointArgs()
+    backend_args = DummyBackendEntrypointArgs()
     with pytest.raises(TypeError):
-        plugins.set_missing_parameters({"engine": backend})
+        plugins.set_missing_parameters({"engine": backend_args})
 
 
 @mock.patch(
-    "pkg_resources.EntryPoint.load",
+    f"{importlib_metadata_mock}.EntryPoint.load",
     mock.MagicMock(return_value=DummyBackendEntrypoint1),
 )
-def test_build_engines():
-    dummy_pkg_entrypoint = pkg_resources.EntryPoint.parse(
-        "cfgrib = xarray.tests.test_plugins:backend_1"
+def test_build_engines() -> None:
+    dummy_pkg_entrypoint = EntryPoint(
+        "cfgrib", "xarray.tests.test_plugins:backend_1", "xarray_backends"
     )
     backend_entrypoints = plugins.build_engines([dummy_pkg_entrypoint])
 
@@ -120,17 +137,13 @@ def test_build_engines():
 
 
 @mock.patch(
-    "pkg_resources.EntryPoint.load",
+    f"{importlib_metadata_mock}.EntryPoint.load",
     mock.MagicMock(return_value=DummyBackendEntrypoint1),
 )
-def test_build_engines_sorted():
+def test_build_engines_sorted() -> None:
     dummy_pkg_entrypoints = [
-        pkg_resources.EntryPoint.parse(
-            "dummy2 = xarray.tests.test_plugins:backend_1",
-        ),
-        pkg_resources.EntryPoint.parse(
-            "dummy1 = xarray.tests.test_plugins:backend_1",
-        ),
+        EntryPoint("dummy2", "xarray.tests.test_plugins:backend_1", "xarray.backends"),
+        EntryPoint("dummy1", "xarray.tests.test_plugins:backend_1", "xarray.backends"),
     ]
     backend_entrypoints = plugins.build_engines(dummy_pkg_entrypoints)
     backend_entrypoints = list(backend_entrypoints)
@@ -146,3 +159,27 @@ def test_build_engines_sorted():
 
     assert set(indices) < {0, -1}
     assert list(backend_entrypoints) == sorted(backend_entrypoints)
+
+
+@mock.patch(
+    "xarray.backends.plugins.list_engines",
+    mock.MagicMock(return_value={"dummy": DummyBackendEntrypointArgs()}),
+)
+def test_no_matching_engine_found() -> None:
+    with pytest.raises(ValueError, match=r"did not find a match in any"):
+        plugins.guess_engine("not-valid")
+
+    with pytest.raises(ValueError, match=r"found the following matches with the input"):
+        plugins.guess_engine("foo.nc")
+
+
+@mock.patch(
+    "xarray.backends.plugins.list_engines",
+    mock.MagicMock(return_value={}),
+)
+def test_engines_not_installed() -> None:
+    with pytest.raises(ValueError, match=r"xarray is unable to open"):
+        plugins.guess_engine("not-valid")
+
+    with pytest.raises(ValueError, match=r"found the following matches with the input"):
+        plugins.guess_engine("foo.nc")
