@@ -15,11 +15,11 @@ from xarray.core.indexing import (
     BasicIndexer,
     CopyOnWriteArray,
     DaskIndexingAdapter,
-    LazilyOuterIndexedArray,
+    LazilyIndexedArray,
     MemoryCachedArray,
     NumpyIndexingAdapter,
     OuterIndexer,
-    PandasIndexAdapter,
+    PandasIndexingAdapter,
     VectorizedIndexer,
 )
 from xarray.core.pycompat import dask_array_type
@@ -32,8 +32,10 @@ from . import (
     assert_array_equal,
     assert_equal,
     assert_identical,
-    raises_regex,
+    raise_if_dask_computes,
+    requires_cupy,
     requires_dask,
+    requires_pint,
     requires_sparse,
     source_ndarray,
 )
@@ -47,6 +49,11 @@ _PAD_XR_NP_ARGS = [
 ]
 
 
+@pytest.fixture
+def var():
+    return Variable(dims=list("xyz"), data=np.random.rand(3, 4, 5))
+
+
 class VariableSubclassobjects:
     def test_properties(self):
         data = 0.5 * np.arange(10)
@@ -119,7 +126,7 @@ class VariableSubclassobjects:
         v_new = v[[True, False, True]]
         assert_identical(v[[0, 2]], v_new)
 
-        with raises_regex(IndexError, "Boolean indexer should"):
+        with pytest.raises(IndexError, match=r"Boolean indexer should"):
             ind = Variable(("a",), [True, False, True])
             v[ind]
 
@@ -225,7 +232,7 @@ class VariableSubclassobjects:
                 return hash(self.item)
 
             def __repr__(self):
-                return "{}(item={!r})".format(type(self).__name__, self.item)
+                return f"{type(self).__name__}(item={self.item!r})"
 
         item = HashableItemWrapper((1, 2, 3))
         x = self.cls("x", [item])
@@ -297,14 +304,14 @@ class VariableSubclassobjects:
     def test_datetime64_valid_range(self):
         data = np.datetime64("1250-01-01", "us")
         pderror = pd.errors.OutOfBoundsDatetime
-        with raises_regex(pderror, "Out of bounds nanosecond"):
+        with pytest.raises(pderror, match=r"Out of bounds nanosecond"):
             self.cls(["t"], [data])
 
     @pytest.mark.xfail(reason="pandas issue 36615")
     def test_timedelta64_valid_range(self):
         data = np.timedelta64("200000", "D")
         pderror = pd.errors.OutOfBoundsTimedelta
-        with raises_regex(pderror, "Out of bounds nanosecond"):
+        with pytest.raises(pderror, match=r"Out of bounds nanosecond"):
             self.cls(["t"], [data])
 
     def test_pandas_data(self):
@@ -336,8 +343,8 @@ class VariableSubclassobjects:
         assert_identical(base_v, 0 + v)
         assert_identical(base_v, v * 1)
         # binary ops with numpy arrays
-        assert_array_equal((v * x).values, x ** 2)
-        assert_array_equal((x * v).values, x ** 2)
+        assert_array_equal((v * x).values, x**2)
+        assert_array_equal((x * v).values, x**2)
         assert_array_equal(v - y, v - 1)
         assert_array_equal(y - v, 1 - v)
         # verify attributes are dropped
@@ -351,7 +358,7 @@ class VariableSubclassobjects:
         assert_array_equal((v * w).values, x * y)
 
         # something complicated
-        assert_array_equal((v ** 2 * w - 1 + x).values, x ** 2 * y - 1 + x)
+        assert_array_equal((v**2 * w - 1 + x).values, x**2 * y - 1 + x)
         # make sure dtype is preserved (for Index objects)
         assert float == (+v).dtype
         assert float == (+v).values.dtype
@@ -454,7 +461,7 @@ class VariableSubclassobjects:
         assert_identical(
             Variable(["b", "a"], np.array([x, y])), Variable.concat((v, w), "b")
         )
-        with raises_regex(ValueError, "Variable has dimensions"):
+        with pytest.raises(ValueError, match=r"Variable has dimensions"):
             Variable.concat([v, Variable(["c"], y)], "b")
         # test indexers
         actual = Variable.concat(
@@ -469,7 +476,7 @@ class VariableSubclassobjects:
         assert_identical(v, Variable.concat([v[:1], v[1:]], "time"))
         # test dimension order
         assert_identical(v, Variable.concat([v[:, :5], v[:, 5:]], "x"))
-        with raises_regex(ValueError, "all input arrays must have"):
+        with pytest.raises(ValueError, match=r"all input arrays must have"):
             Variable.concat([v[:, 0], v[:, 1:]], "x")
 
     def test_concat_attrs(self):
@@ -530,7 +537,7 @@ class VariableSubclassobjects:
         v = self.cls("x", midx)
         for deep in [True, False]:
             w = v.copy(deep=deep)
-            assert isinstance(w._data, PandasIndexAdapter)
+            assert isinstance(w._data, PandasIndexingAdapter)
             assert isinstance(w.to_index(), pd.MultiIndex)
             assert_array_equal(v._data.array, w._data.array)
 
@@ -545,7 +552,7 @@ class VariableSubclassobjects:
     def test_copy_with_data_errors(self):
         orig = Variable(("x", "y"), [[1.5, 2.0], [3.1, 4.3]], {"foo": "bar"})
         new_data = [2.5, 5.0]
-        with raises_regex(ValueError, "must match shape of object"):
+        with pytest.raises(ValueError, match=r"must match shape of object"):
             orig.copy(data=new_data)
 
     def test_copy_index_with_data(self):
@@ -558,11 +565,11 @@ class VariableSubclassobjects:
     def test_copy_index_with_data_errors(self):
         orig = IndexVariable("x", np.arange(5))
         new_data = np.arange(5, 20)
-        with raises_regex(ValueError, "must match shape of object"):
+        with pytest.raises(ValueError, match=r"must match shape of object"):
             orig.copy(data=new_data)
-        with raises_regex(ValueError, "Cannot assign to the .data"):
+        with pytest.raises(ValueError, match=r"Cannot assign to the .data"):
             orig.data = new_data
-        with raises_regex(ValueError, "Cannot assign to the .values"):
+        with pytest.raises(ValueError, match=r"Cannot assign to the .values"):
             orig.values = new_data
 
     def test_replace(self):
@@ -658,12 +665,12 @@ class VariableSubclassobjects:
 
         # with boolean variable with wrong shape
         ind = np.array([True, False])
-        with raises_regex(IndexError, "Boolean array size 2 is "):
+        with pytest.raises(IndexError, match=r"Boolean array size 2 is "):
             v[Variable(("a", "b"), [[0, 1]]), ind]
 
         # boolean indexing with different dimension
         ind = Variable(["a"], [True, False, False])
-        with raises_regex(IndexError, "Boolean indexer should be"):
+        with pytest.raises(IndexError, match=r"Boolean indexer should be"):
             v[dict(y=ind)]
 
     def test_getitem_uint_1d(self):
@@ -793,21 +800,21 @@ class VariableSubclassobjects:
     def test_getitem_error(self):
         v = self.cls(["x", "y"], [[0, 1, 2], [3, 4, 5]])
 
-        with raises_regex(IndexError, "labeled multi-"):
+        with pytest.raises(IndexError, match=r"labeled multi-"):
             v[[[0, 1], [1, 2]]]
 
         ind_x = Variable(["a"], [0, 1, 1])
         ind_y = Variable(["a"], [0, 1])
-        with raises_regex(IndexError, "Dimensions of indexers "):
+        with pytest.raises(IndexError, match=r"Dimensions of indexers "):
             v[ind_x, ind_y]
 
         ind = Variable(["a", "b"], [[True, False], [False, True]])
-        with raises_regex(IndexError, "2-dimensional boolean"):
+        with pytest.raises(IndexError, match=r"2-dimensional boolean"):
             v[dict(x=ind)]
 
         v = Variable(["x", "y", "z"], np.arange(60).reshape(3, 4, 5))
         ind = Variable(["x"], [0, 1])
-        with raises_regex(IndexError, "Dimensions of indexers mis"):
+        with pytest.raises(IndexError, match=r"Dimensions of indexers mis"):
             v[:, ind]
 
     @pytest.mark.parametrize(
@@ -873,26 +880,100 @@ class VariableSubclassobjects:
         )
         assert_array_equal(actual, expected)
 
-    def test_rolling_window(self):
+    @pytest.mark.parametrize("d, w", (("x", 3), ("y", 5)))
+    def test_rolling_window(self, d, w):
         # Just a working test. See test_nputils for the algorithm validation
         v = self.cls(["x", "y", "z"], np.arange(40 * 30 * 2).reshape(40, 30, 2))
-        for (d, w) in [("x", 3), ("y", 5)]:
-            v_rolling = v.rolling_window(d, w, d + "_window")
-            assert v_rolling.dims == ("x", "y", "z", d + "_window")
-            assert v_rolling.shape == v.shape + (w,)
+        v_rolling = v.rolling_window(d, w, d + "_window")
+        assert v_rolling.dims == ("x", "y", "z", d + "_window")
+        assert v_rolling.shape == v.shape + (w,)
 
-            v_rolling = v.rolling_window(d, w, d + "_window", center=True)
-            assert v_rolling.dims == ("x", "y", "z", d + "_window")
-            assert v_rolling.shape == v.shape + (w,)
+        v_rolling = v.rolling_window(d, w, d + "_window", center=True)
+        assert v_rolling.dims == ("x", "y", "z", d + "_window")
+        assert v_rolling.shape == v.shape + (w,)
 
-            # dask and numpy result should be the same
-            v_loaded = v.load().rolling_window(d, w, d + "_window", center=True)
-            assert_array_equal(v_rolling, v_loaded)
+        # dask and numpy result should be the same
+        v_loaded = v.load().rolling_window(d, w, d + "_window", center=True)
+        assert_array_equal(v_rolling, v_loaded)
 
-            # numpy backend should not be over-written
-            if isinstance(v._data, np.ndarray):
-                with pytest.raises(ValueError):
-                    v_loaded[0] = 1.0
+        # numpy backend should not be over-written
+        if isinstance(v._data, np.ndarray):
+            with pytest.raises(ValueError):
+                v_loaded[0] = 1.0
+
+    def test_rolling_1d(self):
+        x = self.cls("x", np.array([1, 2, 3, 4], dtype=float))
+
+        kwargs = dict(dim="x", window=3, window_dim="xw")
+        actual = x.rolling_window(**kwargs, center=True, fill_value=np.nan)
+        expected = Variable(
+            ("x", "xw"),
+            np.array(
+                [[np.nan, 1, 2], [1, 2, 3], [2, 3, 4], [3, 4, np.nan]], dtype=float
+            ),
+        )
+        assert_equal(actual, expected)
+
+        actual = x.rolling_window(**kwargs, center=False, fill_value=0.0)
+        expected = self.cls(
+            ("x", "xw"),
+            np.array([[0, 0, 1], [0, 1, 2], [1, 2, 3], [2, 3, 4]], dtype=float),
+        )
+        assert_equal(actual, expected)
+
+        x = self.cls(("y", "x"), np.stack([x, x * 1.1]))
+        actual = x.rolling_window(**kwargs, center=False, fill_value=0.0)
+        expected = self.cls(
+            ("y", "x", "xw"), np.stack([expected.data, expected.data * 1.1], axis=0)
+        )
+        assert_equal(actual, expected)
+
+    @pytest.mark.parametrize("center", [[True, True], [False, False]])
+    @pytest.mark.parametrize("dims", [("x", "y"), ("y", "z"), ("z", "x")])
+    def test_nd_rolling(self, center, dims):
+        x = self.cls(
+            ("x", "y", "z"),
+            np.arange(7 * 6 * 8).reshape(7, 6, 8).astype(float),
+        )
+        window = [3, 3]
+        actual = x.rolling_window(
+            dim=dims,
+            window=window,
+            window_dim=[f"{k}w" for k in dims],
+            center=center,
+            fill_value=np.nan,
+        )
+        expected = x
+        for dim, win, cent in zip(dims, window, center):
+            expected = expected.rolling_window(
+                dim=dim,
+                window=win,
+                window_dim=f"{dim}w",
+                center=cent,
+                fill_value=np.nan,
+            )
+        assert_equal(actual, expected)
+
+    @pytest.mark.parametrize(
+        ("dim, window, window_dim, center"),
+        [
+            ("x", [3, 3], "x_w", True),
+            ("x", 3, ("x_w", "x_w"), True),
+            ("x", 3, "x_w", [True, True]),
+        ],
+    )
+    def test_rolling_window_errors(self, dim, window, window_dim, center):
+        x = self.cls(
+            ("x", "y", "z"),
+            np.arange(7 * 6 * 8).reshape(7, 6, 8).astype(float),
+        )
+        with pytest.raises(ValueError):
+            x.rolling_window(
+                dim=dim,
+                window=window,
+                window_dim=window_dim,
+                center=center,
+            )
 
 
 class TestVariable(VariableSubclassobjects):
@@ -938,7 +1019,7 @@ class TestVariable(VariableSubclassobjects):
             assert v.values.dtype == np.dtype("datetime64[ns]")
 
     def test_timedelta64_conversion_scalar(self):
-        expected = np.timedelta64(24 * 60 * 60 * 10 ** 9, "ns")
+        expected = np.timedelta64(24 * 60 * 60 * 10**9, "ns")
         for values in [
             np.timedelta64(1, "D"),
             pd.Timedelta("1 day"),
@@ -967,7 +1048,7 @@ class TestVariable(VariableSubclassobjects):
         for td in [pd.to_timedelta("1s"), np.timedelta64(1, "s")]:
             v = Variable([], td)
             assert v.dtype == np.dtype("timedelta64[ns]")
-            assert v.values == np.timedelta64(10 ** 9, "ns")
+            assert v.values == np.timedelta64(10**9, "ns")
 
     def test_equals_and_identical(self):
         d = np.random.rand(10, 3)
@@ -1054,11 +1135,11 @@ class TestVariable(VariableSubclassobjects):
         )
         assert_identical(expected_extra, as_variable(xarray_tuple))
 
-        with raises_regex(TypeError, "tuple of form"):
+        with pytest.raises(TypeError, match=r"tuple of form"):
             as_variable(tuple(data))
-        with raises_regex(ValueError, "tuple of form"):  # GH1016
+        with pytest.raises(ValueError, match=r"tuple of form"):  # GH1016
             as_variable(("five", "six", "seven"))
-        with raises_regex(TypeError, "without an explicit list of dimensions"):
+        with pytest.raises(TypeError, match=r"without an explicit list of dimensions"):
             as_variable(data)
 
         actual = as_variable(data, name="x")
@@ -1070,9 +1151,9 @@ class TestVariable(VariableSubclassobjects):
 
         data = np.arange(9).reshape((3, 3))
         expected = Variable(("x", "y"), data)
-        with raises_regex(ValueError, "without explicit dimension names"):
+        with pytest.raises(ValueError, match=r"without explicit dimension names"):
             as_variable(data, name="x")
-        with raises_regex(ValueError, "has more than 1-dimension"):
+        with pytest.raises(ValueError, match=r"has more than 1-dimension"):
             as_variable(expected, name="x")
 
         # test datetime, timedelta conversion
@@ -1081,7 +1162,7 @@ class TestVariable(VariableSubclassobjects):
         td = np.array([timedelta(days=x) for x in range(10)])
         assert as_variable(td, "time").dtype.kind == "m"
 
-        with pytest.warns(DeprecationWarning):
+        with pytest.raises(TypeError):
             as_variable(("x", DataArray([])))
 
     def test_repr(self):
@@ -1098,12 +1179,12 @@ class TestVariable(VariableSubclassobjects):
         assert expected == repr(v)
 
     def test_repr_lazy_data(self):
-        v = Variable("x", LazilyOuterIndexedArray(np.arange(2e5)))
+        v = Variable("x", LazilyIndexedArray(np.arange(2e5)))
         assert "200000 values with dtype" in repr(v)
-        assert isinstance(v._data, LazilyOuterIndexedArray)
+        assert isinstance(v._data, LazilyIndexedArray)
 
     def test_detect_indexer_type(self):
-        """ Tests indexer type was correctly detected. """
+        """Tests indexer type was correctly detected."""
         data = np.random.random((10, 11))
         v = Variable(["x", "y"], data)
 
@@ -1200,7 +1281,7 @@ class TestVariable(VariableSubclassobjects):
         # test iteration
         for n, item in enumerate(v):
             assert_identical(Variable(["y"], data[n]), item)
-        with raises_regex(TypeError, "iteration over a 0-d"):
+        with pytest.raises(TypeError, match=r"iteration over a 0-d"):
             iter(Variable([], 0))
         # test setting
         v.values[:] = 0
@@ -1274,9 +1355,9 @@ class TestVariable(VariableSubclassobjects):
         assert_identical(v.isel(x=0), v[:, 0])
         assert_identical(v.isel(x=[0, 2]), v[:, [0, 2]])
         assert_identical(v.isel(time=[]), v[[]])
-        with raises_regex(
+        with pytest.raises(
             ValueError,
-            r"Dimensions {'not_a_dim'} do not exist. Expected one or more of "
+            match=r"Dimensions {'not_a_dim'} do not exist. Expected one or more of "
             r"\('time', 'x'\)",
         ):
             v.isel(not_a_dim=0)
@@ -1329,7 +1410,7 @@ class TestVariable(VariableSubclassobjects):
         assert_identical(expected, v.shift(x=5, fill_value=fill_value))
         assert_identical(expected, v.shift(x=6, fill_value=fill_value))
 
-        with raises_regex(ValueError, "dimension"):
+        with pytest.raises(ValueError, match=r"dimension"):
             v.shift(z=0)
 
         v = Variable("x", [1, 2, 3, 4, 5], {"foo": "bar"})
@@ -1358,7 +1439,7 @@ class TestVariable(VariableSubclassobjects):
         assert_identical(expected, v.roll(x=2))
         assert_identical(expected, v.roll(x=-3))
 
-        with raises_regex(ValueError, "dimension"):
+        with pytest.raises(ValueError, match=r"dimension"):
             v.roll(z=0)
 
     def test_roll_consistency(self):
@@ -1387,6 +1468,20 @@ class TestVariable(VariableSubclassobjects):
         w3 = Variable(["b", "c", "d", "a"], np.einsum("abcd->bcda", x))
         assert_identical(w, w3.transpose("a", "b", "c", "d"))
 
+        # test missing dimension, raise error
+        with pytest.raises(ValueError):
+            v.transpose(..., "not_a_dim")
+
+        # test missing dimension, ignore error
+        actual = v.transpose(..., "not_a_dim", missing_dims="ignore")
+        expected_ell = v.transpose(...)
+        assert_identical(expected_ell, actual)
+
+        # test missing dimension, raise warning
+        with pytest.warns(UserWarning):
+            v.transpose(..., "not_a_dim", missing_dims="warn")
+            assert_identical(expected_ell, actual)
+
     def test_transpose_0d(self):
         for value in [
             3.5,
@@ -1411,7 +1506,7 @@ class TestVariable(VariableSubclassobjects):
         v = Variable(["x", "y"], [[1, 2]])
         assert_identical(Variable(["y"], [1, 2]), v.squeeze())
         assert_identical(Variable(["y"], [1, 2]), v.squeeze("x"))
-        with raises_regex(ValueError, "cannot select a dimension"):
+        with pytest.raises(ValueError, match=r"cannot select a dimension"):
             v.squeeze("y")
 
     def test_get_axis_num(self):
@@ -1420,7 +1515,7 @@ class TestVariable(VariableSubclassobjects):
         assert v.get_axis_num(["x"]) == (0,)
         assert v.get_axis_num(["x", "y"]) == (0, 1)
         assert v.get_axis_num(["z", "y", "x"]) == (2, 1, 0)
-        with raises_regex(ValueError, "not found in array dim"):
+        with pytest.raises(ValueError, match=r"not found in array dim"):
             v.get_axis_num("foobar")
 
     def test_set_dims(self):
@@ -1441,7 +1536,7 @@ class TestVariable(VariableSubclassobjects):
         expected = v
         assert_identical(actual, expected)
 
-        with raises_regex(ValueError, "must be a superset"):
+        with pytest.raises(ValueError, match=r"must be a superset"):
             v.set_dims(["z"])
 
     def test_set_dims_object_dtype(self):
@@ -1473,9 +1568,9 @@ class TestVariable(VariableSubclassobjects):
     def test_stack_errors(self):
         v = Variable(["x", "y"], [[0, 1], [2, 3]], {"foo": "bar"})
 
-        with raises_regex(ValueError, "invalid existing dim"):
+        with pytest.raises(ValueError, match=r"invalid existing dim"):
             v.stack(z=("x1",))
-        with raises_regex(ValueError, "cannot create a new dim"):
+        with pytest.raises(ValueError, match=r"cannot create a new dim"):
             v.stack(x=("x",))
 
     def test_unstack(self):
@@ -1494,11 +1589,11 @@ class TestVariable(VariableSubclassobjects):
 
     def test_unstack_errors(self):
         v = Variable("z", [0, 1, 2, 3])
-        with raises_regex(ValueError, "invalid existing dim"):
+        with pytest.raises(ValueError, match=r"invalid existing dim"):
             v.unstack(foo={"x": 4})
-        with raises_regex(ValueError, "cannot create a new dim"):
+        with pytest.raises(ValueError, match=r"cannot create a new dim"):
             v.stack(z=("z",))
-        with raises_regex(ValueError, "the product of the new dim"):
+        with pytest.raises(ValueError, match=r"the product of the new dim"):
             v.unstack(z={"x": 5})
 
     def test_unstack_2d(self):
@@ -1543,9 +1638,9 @@ class TestVariable(VariableSubclassobjects):
         a = Variable(["x"], np.arange(10))
         b = Variable(["x"], np.arange(5))
         c = Variable(["x", "x"], np.arange(100).reshape(10, 10))
-        with raises_regex(ValueError, "mismatched lengths"):
+        with pytest.raises(ValueError, match=r"mismatched lengths"):
             a + b
-        with raises_regex(ValueError, "duplicate dimensions"):
+        with pytest.raises(ValueError, match=r"duplicate dimensions"):
             a + c
 
     def test_inplace_math(self):
@@ -1558,9 +1653,17 @@ class TestVariable(VariableSubclassobjects):
         assert source_ndarray(v.values) is x
         assert_array_equal(v.values, np.arange(5) + 1)
 
-        with raises_regex(ValueError, "dimensions cannot change"):
+        with pytest.raises(ValueError, match=r"dimensions cannot change"):
             v += Variable("y", np.arange(5))
 
+    def test_inplace_math_error(self):
+        x = np.arange(5)
+        v = IndexVariable(["x"], x)
+        with pytest.raises(
+            TypeError, match=r"Values of an IndexVariable are immutable"
+        ):
+            v += 1
+
     def test_reduce(self):
         v = Variable(["x", "y"], self.d, {"ignored": "attributes"})
         assert_identical(v.reduce(np.std, "x"), Variable(["y"], self.d.std(axis=0)))
@@ -1575,9 +1678,26 @@ class TestVariable(VariableSubclassobjects):
         )
         assert_allclose(v.mean("x"), v.reduce(np.mean, "x"))
 
-        with raises_regex(ValueError, "cannot supply both"):
+        with pytest.raises(ValueError, match=r"cannot supply both"):
             v.mean(dim="x", axis=0)
 
+    @requires_bottleneck
+    def test_reduce_use_bottleneck(self, monkeypatch):
+        def raise_if_called(*args, **kwargs):
+            raise RuntimeError("should not have been called")
+
+        import bottleneck as bn
+
+        monkeypatch.setattr(bn, "nanmin", raise_if_called)
+
+        v = Variable("x", [0.0, np.nan, 1.0])
+        with pytest.raises(RuntimeError, match="should not have been called"):
+            with set_options(use_bottleneck=True):
+                v.min()
+
+        with set_options(use_bottleneck=False):
+            v.min()
+
     @pytest.mark.parametrize("skipna", [True, False])
     @pytest.mark.parametrize("q", [0.25, [0.50], [0.25, 0.75]])
     @pytest.mark.parametrize(
@@ -1605,7 +1725,7 @@ class TestVariable(VariableSubclassobjects):
         v = Variable(["x", "y"], self.d).chunk({"x": 2})
 
         # this checks for ValueError in dask.array.apply_gufunc
-        with raises_regex(ValueError, "consists of multiple chunks"):
+        with pytest.raises(ValueError, match=r"consists of multiple chunks"):
             v.quantile(0.5, dim="x")
 
     @pytest.mark.parametrize("q", [-0.1, 1.1, [2], [0.25, 2]])
@@ -1613,16 +1733,24 @@ class TestVariable(VariableSubclassobjects):
         v = Variable(["x", "y"], self.d)
 
         # escape special characters
-        with raises_regex(ValueError, r"Quantiles must be in the range \[0, 1\]"):
+        with pytest.raises(
+            ValueError, match=r"Quantiles must be in the range \[0, 1\]"
+        ):
             v.quantile(q, dim="x")
 
     @requires_dask
     @requires_bottleneck
     def test_rank_dask_raises(self):
         v = Variable(["x"], [3.0, 1.0, np.nan, 2.0, 4.0]).chunk(2)
-        with raises_regex(TypeError, "arrays stored as dask"):
+        with pytest.raises(TypeError, match=r"arrays stored as dask"):
             v.rank("x")
 
+    def test_rank_use_bottleneck(self):
+        v = Variable(["x"], [3.0, 1.0, np.nan, 2.0, 4.0])
+        with set_options(use_bottleneck=False):
+            with pytest.raises(RuntimeError):
+                v.rank("x")
+
     @requires_bottleneck
     def test_rank(self):
         import bottleneck as bn
@@ -1646,7 +1774,7 @@ class TestVariable(VariableSubclassobjects):
         v_expect = Variable(["x"], [0.75, 0.25, np.nan, 0.5, 1.0])
         assert_equal(v.rank("x", pct=True), v_expect)
         # invalid dim
-        with raises_regex(ValueError, "not found"):
+        with pytest.raises(ValueError, match=r"not found"):
             v.rank("y")
 
     def test_big_endian_reduce(self):
@@ -1853,7 +1981,7 @@ class TestVariable(VariableSubclassobjects):
         expected = Variable(["x", "y"], [[0, 0], [0, 0], [1, 1]])
         assert_identical(expected, v)
 
-        with raises_regex(ValueError, "shape mismatch"):
+        with pytest.raises(ValueError, match=r"shape mismatch"):
             v[ind, ind] = np.zeros((1, 2, 1))
 
         v = Variable(["x", "y"], [[0, 3, 2], [3, 4, 5]])
@@ -2010,6 +2138,35 @@ class TestVariableWithDask(VariableSubclassobjects):
             self.cls(("x", "y"), [[0, -1], [-1, 2]]),
         )
 
+    @pytest.mark.parametrize("dim", ["x", "y"])
+    @pytest.mark.parametrize("window", [3, 8, 11])
+    @pytest.mark.parametrize("center", [True, False])
+    def test_dask_rolling(self, dim, window, center):
+        import dask
+        import dask.array as da
+
+        dask.config.set(scheduler="single-threaded")
+
+        x = Variable(("x", "y"), np.array(np.random.randn(100, 40), dtype=float))
+        dx = Variable(("x", "y"), da.from_array(x, chunks=[(6, 30, 30, 20, 14), 8]))
+
+        expected = x.rolling_window(
+            dim, window, "window", center=center, fill_value=np.nan
+        )
+        with raise_if_dask_computes():
+            actual = dx.rolling_window(
+                dim, window, "window", center=center, fill_value=np.nan
+            )
+        assert isinstance(actual.data, da.Array)
+        assert actual.shape == expected.shape
+        assert_equal(actual, expected)
+
+    @pytest.mark.xfail(
+        reason="https://github.com/pydata/xarray/issues/6209#issuecomment-1025116203"
+    )
+    def test_multiindex(self):
+        super().test_multiindex()
+
 
 @requires_sparse
 class TestVariableWithSparse:
@@ -2026,7 +2183,7 @@ class TestIndexVariable(VariableSubclassobjects):
     cls = staticmethod(IndexVariable)
 
     def test_init(self):
-        with raises_regex(ValueError, "must be 1-dimensional"):
+        with pytest.raises(ValueError, match=r"must be 1-dimensional"):
             IndexVariable((), 0)
 
     def test_to_index(self):
@@ -2041,12 +2198,12 @@ class TestIndexVariable(VariableSubclassobjects):
 
     def test_data(self):
         x = IndexVariable("x", np.arange(3.0))
-        assert isinstance(x._data, PandasIndexAdapter)
+        assert isinstance(x._data, PandasIndexingAdapter)
         assert isinstance(x.data, np.ndarray)
         assert float == x.dtype
         assert_array_equal(np.arange(3), x)
         assert float == x.values.dtype
-        with raises_regex(TypeError, "cannot be modified"):
+        with pytest.raises(TypeError, match=r"cannot be modified"):
             x[:] = 0
 
     def test_name(self):
@@ -2073,7 +2230,7 @@ class TestIndexVariable(VariableSubclassobjects):
         level_1 = IndexVariable("x", midx.get_level_values("level_1"))
         assert_identical(x.get_level_variable("level_1"), level_1)
 
-        with raises_regex(ValueError, "has no MultiIndex"):
+        with pytest.raises(ValueError, match=r"has no MultiIndex"):
             IndexVariable("y", [10.0]).get_level_variable("level")
 
     def test_concat_periods(self):
@@ -2121,23 +2278,23 @@ class TestIndexVariable(VariableSubclassobjects):
 
     # These tests make use of multi-dimensional variables, which are not valid
     # IndexVariable objects:
-    @pytest.mark.xfail
+    @pytest.mark.skip
     def test_getitem_error(self):
         super().test_getitem_error()
 
-    @pytest.mark.xfail
+    @pytest.mark.skip
     def test_getitem_advanced(self):
         super().test_getitem_advanced()
 
-    @pytest.mark.xfail
+    @pytest.mark.skip
     def test_getitem_fancy(self):
         super().test_getitem_fancy()
 
-    @pytest.mark.xfail
+    @pytest.mark.skip
     def test_getitem_uint(self):
         super().test_getitem_fancy()
 
-    @pytest.mark.xfail
+    @pytest.mark.skip
     @pytest.mark.parametrize(
         "mode",
         [
@@ -2156,23 +2313,34 @@ class TestIndexVariable(VariableSubclassobjects):
     def test_pad(self, mode, xr_arg, np_arg):
         super().test_pad(mode, xr_arg, np_arg)
 
-    @pytest.mark.xfail
-    @pytest.mark.parametrize("xr_arg, np_arg", _PAD_XR_NP_ARGS)
+    @pytest.mark.skip
     def test_pad_constant_values(self, xr_arg, np_arg):
         super().test_pad_constant_values(xr_arg, np_arg)
 
-    @pytest.mark.xfail
+    @pytest.mark.skip
     def test_rolling_window(self):
         super().test_rolling_window()
 
-    @pytest.mark.xfail
+    @pytest.mark.skip
+    def test_rolling_1d(self):
+        super().test_rolling_1d()
+
+    @pytest.mark.skip
+    def test_nd_rolling(self):
+        super().test_nd_rolling()
+
+    @pytest.mark.skip
+    def test_rolling_window_errors(self):
+        super().test_rolling_window_errors()
+
+    @pytest.mark.skip
     def test_coarsen_2d(self):
         super().test_coarsen_2d()
 
 
 class TestAsCompatibleData:
     def test_unchanged_types(self):
-        types = (np.asarray, PandasIndexAdapter, LazilyOuterIndexedArray)
+        types = (np.asarray, PandasIndexingAdapter, LazilyIndexedArray)
         for t in types:
             for data in [
                 np.arange(3),
@@ -2245,7 +2413,7 @@ class TestAsCompatibleData:
         assert_identical(expect, full_like(orig, True, dtype=bool))
 
         # raise error on non-scalar fill_value
-        with raises_regex(ValueError, "must be scalar"):
+        with pytest.raises(ValueError, match=r"must be scalar"):
             full_like(orig, [1.0, 2.0])
 
         with pytest.raises(ValueError, match="'dtype' cannot be dict-like"):
@@ -2331,7 +2499,7 @@ def test_raise_no_warning_for_nan_in_binary_ops():
 
 
 class TestBackendIndexing:
-    """    Make sure all the array wrappers can be indexed. """
+    """Make sure all the array wrappers can be indexed."""
 
     @pytest.fixture(autouse=True)
     def setUp(self):
@@ -2350,24 +2518,24 @@ class TestBackendIndexing:
         self.check_orthogonal_indexing(v)
         self.check_vectorized_indexing(v)
         # could not doubly wrapping
-        with raises_regex(TypeError, "NumpyIndexingAdapter only wraps "):
+        with pytest.raises(TypeError, match=r"NumpyIndexingAdapter only wraps "):
             v = Variable(
                 dims=("x", "y"), data=NumpyIndexingAdapter(NumpyIndexingAdapter(self.d))
             )
 
-    def test_LazilyOuterIndexedArray(self):
-        v = Variable(dims=("x", "y"), data=LazilyOuterIndexedArray(self.d))
+    def test_LazilyIndexedArray(self):
+        v = Variable(dims=("x", "y"), data=LazilyIndexedArray(self.d))
         self.check_orthogonal_indexing(v)
         self.check_vectorized_indexing(v)
         # doubly wrapping
         v = Variable(
             dims=("x", "y"),
-            data=LazilyOuterIndexedArray(LazilyOuterIndexedArray(self.d)),
+            data=LazilyIndexedArray(LazilyIndexedArray(self.d)),
         )
         self.check_orthogonal_indexing(v)
         # hierarchical wrapping
         v = Variable(
-            dims=("x", "y"), data=LazilyOuterIndexedArray(NumpyIndexingAdapter(self.d))
+            dims=("x", "y"), data=LazilyIndexedArray(NumpyIndexingAdapter(self.d))
         )
         self.check_orthogonal_indexing(v)
 
@@ -2376,9 +2544,7 @@ class TestBackendIndexing:
         self.check_orthogonal_indexing(v)
         self.check_vectorized_indexing(v)
         # doubly wrapping
-        v = Variable(
-            dims=("x", "y"), data=CopyOnWriteArray(LazilyOuterIndexedArray(self.d))
-        )
+        v = Variable(dims=("x", "y"), data=CopyOnWriteArray(LazilyIndexedArray(self.d)))
         self.check_orthogonal_indexing(v)
         self.check_vectorized_indexing(v)
 
@@ -2403,3 +2569,92 @@ class TestBackendIndexing:
         v = Variable(dims=("x", "y"), data=CopyOnWriteArray(DaskIndexingAdapter(da)))
         self.check_orthogonal_indexing(v)
         self.check_vectorized_indexing(v)
+
+
+def test_clip(var):
+    # Copied from test_dataarray (would there be a way to combine the tests?)
+    result = var.clip(min=0.5)
+    assert result.min(...) >= 0.5
+
+    result = var.clip(max=0.5)
+    assert result.max(...) <= 0.5
+
+    result = var.clip(min=0.25, max=0.75)
+    assert result.min(...) >= 0.25
+    assert result.max(...) <= 0.75
+
+    result = var.clip(min=var.mean("x"), max=var.mean("z"))
+    assert result.dims == var.dims
+    assert_array_equal(
+        result.data,
+        np.clip(
+            var.data,
+            var.mean("x").data[np.newaxis, :, :],
+            var.mean("z").data[:, :, np.newaxis],
+        ),
+    )
+
+
+@pytest.mark.parametrize("Var", [Variable, IndexVariable])
+class TestNumpyCoercion:
+    def test_from_numpy(self, Var):
+        v = Var("x", [1, 2, 3])
+
+        assert_identical(v.as_numpy(), v)
+        np.testing.assert_equal(v.to_numpy(), np.array([1, 2, 3]))
+
+    @requires_dask
+    def test_from_dask(self, Var):
+        v = Var("x", [1, 2, 3])
+        v_chunked = v.chunk(1)
+
+        assert_identical(v_chunked.as_numpy(), v.compute())
+        np.testing.assert_equal(v.to_numpy(), np.array([1, 2, 3]))
+
+    @requires_pint
+    def test_from_pint(self, Var):
+        from pint import Quantity
+
+        arr = np.array([1, 2, 3])
+        v = Var("x", Quantity(arr, units="m"))
+
+        assert_identical(v.as_numpy(), Var("x", arr))
+        np.testing.assert_equal(v.to_numpy(), arr)
+
+    @requires_sparse
+    def test_from_sparse(self, Var):
+        if Var is IndexVariable:
+            pytest.skip("Can't have 2D IndexVariables")
+
+        import sparse
+
+        arr = np.diagflat([1, 2, 3])
+        sparr = sparse.COO(coords=[[0, 1, 2], [0, 1, 2]], data=[1, 2, 3])
+        v = Variable(["x", "y"], sparr)
+
+        assert_identical(v.as_numpy(), Variable(["x", "y"], arr))
+        np.testing.assert_equal(v.to_numpy(), arr)
+
+    @requires_cupy
+    def test_from_cupy(self, Var):
+        import cupy as cp
+
+        arr = np.array([1, 2, 3])
+        v = Var("x", cp.array(arr))
+
+        assert_identical(v.as_numpy(), Var("x", arr))
+        np.testing.assert_equal(v.to_numpy(), arr)
+
+    @requires_dask
+    @requires_pint
+    def test_from_pint_wrapping_dask(self, Var):
+        import dask
+        from pint import Quantity
+
+        arr = np.array([1, 2, 3])
+        d = dask.array.from_array(np.array([1, 2, 3]))
+        v = Var("x", Quantity(d, units="m"))
+
+        result = v.as_numpy()
+        assert_identical(result, Var("x", arr))
+        np.testing.assert_equal(v.to_numpy(), arr)
